# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections

from ironicclient import exc as ironicexp
import mock
from testtools import matchers

from os_cloud_config import nodes
from os_cloud_config.tests import base


class NodesTest(base.TestCase):

    def _get_node(self):
        return {'cpu': '1', 'memory': '2048', 'disk': '30', 'arch': 'amd64',
                'mac': ['aaa'], 'pm_addr': 'foo.bar', 'pm_user': 'test',
                'pm_password': 'random', 'pm_type': 'pxe_ssh', 'name': 'node1',
                'capabilities': 'num_nics:6'}

    def test_register_list_of_nodes(self):
        nodes_list = ['aaa', 'bbb']
        return_node = nodes_list[0]
        register_func = mock.MagicMock()
        register_func.side_effect = [return_node, ironicexp.Conflict]
        seen = nodes._register_list_of_nodes(register_func, {}, None,
                                             nodes_list, False, 'servicehost',
                                             None, None)
        self.assertEqual(seen, set(nodes_list))

    def test_extract_driver_info_ipmi(self):
        node = self._get_node()
        node["pm_type"] = "ipmi"
        expected = {"ipmi_address": "foo.bar",
                    "ipmi_username": "test",
                    "ipmi_password": "random"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_ipmi_extended(self):
        node = self._get_node()
        node["pm_type"] = "ipmi"
        node["ipmi_bridging"] = "dual"
        node["ipmi_transit_address"] = "0x42"
        node["ipmi_transit_channel"] = "0"
        node["ipmi_target_address"] = "0x41"
        node["ipmi_target_channel"] = "1"
        node["ipmi_local_address"] = "0"
        expected = {"ipmi_address": "foo.bar",
                    "ipmi_username": "test",
                    "ipmi_password": "random",
                    "ipmi_bridging": "dual",
                    "ipmi_transit_address": "0x42",
                    "ipmi_transit_channel": "0",
                    "ipmi_target_address": "0x41",
                    "ipmi_target_channel": "1",
                    "ipmi_local_address": "0",
                    }
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_pxe_ssh(self):
        node = self._get_node()
        node["pm_type"] = "pxe_ssh"
        expected = {"ssh_address": "foo.bar",
                    "ssh_username": "test",
                    "ssh_key_contents": "random",
                    "ssh_virt_type": "virsh"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_pxe_drac(self):
        node = self._get_node()
        node["pm_type"] = "pxe_drac"
        expected = {"drac_host": "foo.bar",
                    "drac_username": "test",
                    "drac_password": "random"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_pxe_ssh_with_pm_virt_type(self):
        node = self._get_node()
        node["pm_type"] = "pxe_ssh"
        node["pm_virt_type"] = "vbox"
        expected = {"ssh_address": "foo.bar",
                    "ssh_username": "test",
                    "ssh_key_contents": "random",
                    "ssh_virt_type": "vbox"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_pxe_iboot(self):
        node = self._get_node()
        node["pm_type"] = "pxe_iboot_iscsi"
        expected = {"iboot_address": "foo.bar",
                    "iboot_username": "test",
                    "iboot_password": "random"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_pxe_iboot_with_pm_relay_id(self):
        node = self._get_node()
        node["pm_type"] = "pxe_iboot_iscsi"
        node["pm_relay_id"] = "pxe_iboot_id"
        expected = {"iboot_address": "foo.bar",
                    "iboot_username": "test",
                    "iboot_password": "random",
                    "iboot_relay_id": "pxe_iboot_id"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_pxe_iboot_with_pm_port(self):
        node = self._get_node()
        node["pm_type"] = "pxe_iboot_iscsi"
        node["pm_port"] = "8080"
        expected = {"iboot_address": "foo.bar",
                    "iboot_username": "test",
                    "iboot_password": "random",
                    "iboot_port": "8080"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_pxe_ucs(self):
        node = self._get_node()
        node["pm_type"] = "pxe_ucs"
        node["pm_service_profile"] = "foo_profile"
        expected = {"ucs_address": "foo.bar",
                    "ucs_username": "test",
                    "ucs_password": "random",
                    "ucs_service_profile": "foo_profile"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_pxe_irmc(self):
        node = self._get_node()
        node["pm_type"] = "pxe_irmc"
        expected = {"irmc_address": "foo.bar",
                    "irmc_username": "test",
                    "irmc_password": "random"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_pxe_irmc_with_irmc_port(self):
        node = self._get_node()
        node["pm_type"] = "pxe_irmc"
        node["pm_port"] = "443"
        expected = {"irmc_address": "foo.bar",
                    "irmc_username": "test",
                    "irmc_password": "random",
                    "irmc_port": "443"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_pxe_irmc_with_irmc_auth_method(self):
        node = self._get_node()
        node["pm_type"] = "pxe_irmc"
        node["pm_auth_method"] = "baz_auth_method"
        expected = {"irmc_address": "foo.bar",
                    "irmc_username": "test",
                    "irmc_password": "random",
                    "irmc_auth_method": "baz_auth_method"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_pxe_irmc_with_irmc_client_timeout(self):
        node = self._get_node()
        node["pm_type"] = "pxe_irmc"
        node["pm_client_timeout"] = "60"
        expected = {"irmc_address": "foo.bar",
                    "irmc_username": "test",
                    "irmc_password": "random",
                    "irmc_client_timeout": "60"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_pxe_irmc_with_irmc_sensor_method(self):
        node = self._get_node()
        node["pm_type"] = "pxe_irmc"
        node["pm_sensor_method"] = "ipmitool"
        expected = {"irmc_address": "foo.bar",
                    "irmc_username": "test",
                    "irmc_password": "random",
                    "irmc_sensor_method": "ipmitool"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_iscsi_irmc(self):
        node = self._get_node()
        node["pm_type"] = "iscsi_irmc"
        node["pm_deploy_iso"] = "deploy.iso"
        expected = {"irmc_address": "foo.bar",
                    "irmc_username": "test",
                    "irmc_password": "random",
                    "irmc_deploy_iso": "deploy.iso"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_agent_irmc(self):
        node = self._get_node()
        node["pm_type"] = "agent_irmc"
        node["pm_deploy_iso"] = "deploy.iso"
        expected = {"irmc_address": "foo.bar",
                    "irmc_username": "test",
                    "irmc_password": "random",
                    "irmc_deploy_iso": "deploy.iso"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_pxe_ipmi_with_kernel_ramdisk(self):
        node = self._get_node()
        node["pm_type"] = "pxe_ipmi"
        node["kernel_id"] = "kernel-abc"
        node["ramdisk_id"] = "ramdisk-foo"
        expected = {"ipmi_address": "foo.bar",
                    "ipmi_username": "test",
                    "ipmi_password": "random",
                    "deploy_kernel": "kernel-abc",
                    "deploy_ramdisk": "ramdisk-foo"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_pxe_wol(self):
        node = self._get_node()
        node["pm_type"] = "pxe_wol"
        expected = {"wol_host": "foo.bar"}
        self.assertEqual(expected, nodes._extract_driver_info(node))

    def test_extract_driver_info_unknown_type(self):
        node = self._get_node()
        node["pm_type"] = "unknown_type"
        self.assertRaises(ValueError, nodes._extract_driver_info, node)

    def test_register_all_nodes_ironic_no_hw_stats(self):
        node_list = [self._get_node()]

        # Remove the hardware stats from the node dictionary
        node_list[0].pop("cpu")
        node_list[0].pop("memory")
        node_list[0].pop("disk")
        node_list[0].pop("arch")

        # Node properties should be created with empty string values for the
        # hardware statistics
        node_properties = {"capabilities": "num_nics:6"}

        ironic = mock.MagicMock()
        nodes.register_all_nodes('servicehost', node_list, client=ironic)
        pxe_node_driver_info = {"ssh_address": "foo.bar",
                                "ssh_username": "test",
                                "ssh_key_contents": "random",
                                "ssh_virt_type": "virsh"}
        pxe_node = mock.call(driver="pxe_ssh",
                             name='node1',
                             driver_info=pxe_node_driver_info,
                             properties=node_properties)
        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                              address='aaa')
        power_off_call = mock.call(ironic.node.create.return_value.uuid, 'off')
        ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
        ironic.port.create.assert_has_calls([port_call])
        ironic.node.set_power_state.assert_has_calls([power_off_call])

    def test_register_all_nodes_ironic(self):
        node_list = [self._get_node()]
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        ironic = mock.MagicMock()
        nodes.register_all_nodes('servicehost', node_list, client=ironic)
        pxe_node_driver_info = {"ssh_address": "foo.bar",
                                "ssh_username": "test",
                                "ssh_key_contents": "random",
                                "ssh_virt_type": "virsh"}
        pxe_node = mock.call(driver="pxe_ssh",
                             name='node1',
                             driver_info=pxe_node_driver_info,
                             properties=node_properties)
        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                              address='aaa')
        power_off_call = mock.call(ironic.node.create.return_value.uuid, 'off')
        ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
        ironic.port.create.assert_has_calls([port_call])
        ironic.node.set_power_state.assert_has_calls([power_off_call])

    def test_register_all_nodes_ironic_kernel_ramdisk(self):
        node_list = [self._get_node()]
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        ironic = mock.MagicMock()
        glance = mock.MagicMock()
        image = collections.namedtuple('image', ['id'])
        glance.images.find.side_effect = (image('kernel-123'),
                                          image('ramdisk-999'))
        nodes.register_all_nodes('servicehost', node_list, client=ironic,
                                 glance_client=glance, kernel_name='bm-kernel',
                                 ramdisk_name='bm-ramdisk')
        pxe_node_driver_info = {"ssh_address": "foo.bar",
                                "ssh_username": "test",
                                "ssh_key_contents": "random",
                                "ssh_virt_type": "virsh",
                                "deploy_kernel": "kernel-123",
                                "deploy_ramdisk": "ramdisk-999"}
        pxe_node = mock.call(driver="pxe_ssh",
                             name='node1',
                             driver_info=pxe_node_driver_info,
                             properties=node_properties)
        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                              address='aaa')
        power_off_call = mock.call(ironic.node.create.return_value.uuid, 'off')
        ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
        ironic.port.create.assert_has_calls([port_call])
        ironic.node.set_power_state.assert_has_calls([power_off_call])

    @mock.patch('time.sleep')
    def test_register_ironic_node_retry(self, sleep):
        ironic = mock.MagicMock()
        ironic_node = collections.namedtuple('node', ['uuid'])
        side_effect = (ironicexp.ConnectionRefused,
                       ironicexp.ServiceUnavailable, ironic_node('1'))
        ironic.node.create.side_effect = side_effect
        nodes.register_ironic_node(None, self._get_node(), client=ironic)
        sleep.assert_has_calls([mock.call(10), mock.call(10)])
        node_create = mock.call(driver='pxe_ssh',
                                name='node1',
                                driver_info=mock.ANY,
                                properties=mock.ANY)
        ironic.node.create.assert_has_calls([node_create, node_create,
                                             node_create])

    @mock.patch('time.sleep')
    def test_register_ironic_node_failure(self, sleep):
        ironic = mock.MagicMock()
        ironic.node.create.side_effect = ironicexp.ConnectionRefused
        self.assertRaises(ironicexp.ServiceUnavailable,
                          nodes.register_ironic_node, None, self._get_node(),
                          client=ironic)

    def test_register_ironic_node_update(self):
        node = self._get_node()
        ironic = mock.MagicMock()
        node_map = {'mac': {'aaa': 1}}

        def side_effect(*args, **kwargs):
            update_patch = [
                {'path': '/name', 'value': 'node1'},
                {'path': '/driver_info/ssh_key_contents', 'value': 'random'},
                {'path': '/driver_info/ssh_address', 'value': 'foo.bar'},
                {'path': '/properties/memory_mb', 'value': '2048'},
                {'path': '/properties/local_gb', 'value': '30'},
                {'path': '/properties/cpu_arch', 'value': 'amd64'},
                {'path': '/properties/cpus', 'value': '1'},
                {'path': '/properties/capabilities', 'value': 'num_nics:6'},
                {'path': '/driver_info/ssh_username', 'value': 'test'}]
            for key in update_patch:
                key['op'] = 'replace'
            self.assertThat(update_patch,
                            matchers.MatchesSetwise(*(map(matchers.Equals,
                                                          args[1]))))
        ironic.node.update.side_effect = side_effect
        nodes._update_or_register_ironic_node(None, node, node_map,
                                              client=ironic)
        ironic.node.update.assert_called_once_with(
            ironic.node.get.return_value.uuid, mock.ANY)

    def _update_by_type(self, pm_type):
        ironic = mock.MagicMock()
        node_map = {'mac': {}, 'pm_addr': {}}
        node = self._get_node()
        node['pm_type'] = pm_type
        node_map['pm_addr']['foo.bar'] = ironic.node.get.return_value.uuid
        nodes._update_or_register_ironic_node('servicehost', node,
                                              node_map, client=ironic)
        ironic.node.update.assert_called_once_with(
            ironic.node.get.return_value.uuid, mock.ANY)

    def test_update_node_ironic_pxe_ipmitool(self):
        self._update_by_type('pxe_ipmitool')

    def test_update_node_ironic_pxe_drac(self):
        self._update_by_type('pxe_drac')

    def test_update_node_ironic_pxe_ilo(self):
        self._update_by_type('pxe_ilo')

    def test_update_node_ironic_pxe_irmc(self):
        self._update_by_type('pxe_irmc')

    def test_register_ironic_node_update_uppercase_mac(self):
        node = self._get_node()
        node['mac'][0] = node['mac'][0].upper()
        ironic = mock.MagicMock()
        node_map = {'mac': {'aaa': 1}}

        def side_effect(*args, **kwargs):
            update_patch = [
                {'path': '/name', 'value': 'node1'},
                {'path': '/driver_info/ssh_key_contents', 'value': 'random'},
                {'path': '/driver_info/ssh_address', 'value': 'foo.bar'},
                {'path': '/properties/memory_mb', 'value': '2048'},
                {'path': '/properties/local_gb', 'value': '30'},
                {'path': '/properties/cpu_arch', 'value': 'amd64'},
                {'path': '/properties/cpus', 'value': '1'},
                {'path': '/properties/capabilities', 'value': 'num_nics:6'},
                {'path': '/driver_info/ssh_username', 'value': 'test'}]
            for key in update_patch:
                key['op'] = 'replace'
            self.assertThat(update_patch,
                            matchers.MatchesSetwise(*(map(matchers.Equals,
                                                          args[1]))))

        ironic.node.update.side_effect = side_effect
        nodes._update_or_register_ironic_node(None, node, node_map,
                                              client=ironic)
        ironic.node.update.assert_called_once_with(
            ironic.node.get.return_value.uuid, mock.ANY)

    @mock.patch('time.sleep')
    def test_register_ironic_node_update_locked_node(self, sleep):
        node = self._get_node()
        ironic = mock.MagicMock()
        ironic.node.update.side_effect = ironicexp.Conflict
        node_map = {'mac': {'aaa': 1}}
        self.assertRaises(ironicexp.Conflict,
                          nodes._update_or_register_ironic_node, None, node,
                          node_map, client=ironic)

    def test_register_ironic_node_int_values(self):
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        node = self._get_node()
        node['cpu'] = 1
        node['memory'] = 2048
        node['disk'] = 30
        client = mock.MagicMock()
        nodes.register_ironic_node('service_host', node, client=client)
        client.node.create.assert_called_once_with(driver=mock.ANY,
                                                   name='node1',
                                                   properties=node_properties,
                                                   driver_info=mock.ANY)

    def test_register_ironic_node_fake_pxe(self):
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        node = self._get_node()
        for v in ('pm_addr', 'pm_user', 'pm_password'):
            del node[v]
        node['pm_type'] = 'fake_pxe'
        client = mock.MagicMock()
        nodes.register_ironic_node('service_host', node, client=client)
        client.node.create.assert_called_once_with(driver='fake_pxe',
                                                   name='node1',
                                                   properties=node_properties,
                                                   driver_info={})

    def test_register_ironic_node_update_int_values(self):
        node = self._get_node()
        ironic = mock.MagicMock()
        node['cpu'] = 1
        node['memory'] = 2048
        node['disk'] = 30
        node_map = {'mac': {'aaa': 1}}

        def side_effect(*args, **kwargs):
            update_patch = [
                {'path': '/name', 'value': 'node1'},
                {'path': '/driver_info/ssh_key_contents', 'value': 'random'},
                {'path': '/driver_info/ssh_address', 'value': 'foo.bar'},
                {'path': '/properties/memory_mb', 'value': '2048'},
                {'path': '/properties/local_gb', 'value': '30'},
                {'path': '/properties/cpu_arch', 'value': 'amd64'},
                {'path': '/properties/cpus', 'value': '1'},
                {'path': '/properties/capabilities', 'value': 'num_nics:6'},
                {'path': '/driver_info/ssh_username', 'value': 'test'}]
            for key in update_patch:
                key['op'] = 'replace'
            self.assertThat(update_patch,
                            matchers.MatchesSetwise(*(map(matchers.Equals,
                                                      args[1]))))
        ironic.node.update.side_effect = side_effect
        nodes._update_or_register_ironic_node(None, node, node_map,
                                              client=ironic)

    def test_populate_node_mapping_ironic(self):
        client = mock.MagicMock()
        node1 = mock.MagicMock()
        node1.to_dict.return_value = {'uuid': 'abcdef'}
        node2 = mock.MagicMock()
        node2.to_dict.return_value = {'uuid': 'fedcba'}
        ironic_node = collections.namedtuple('node', ['uuid', 'driver',
                                             'driver_info'])
        ironic_port = collections.namedtuple('port', ['address'])
        node1_detail = ironic_node('abcdef', 'pxe_ssh', None)
        node2_detail = ironic_node('fedcba', 'ipmi',
                                   {'ipmi_address': '10.0.1.2'})
        client.node.get.side_effect = (node1_detail, node2_detail)
        client.node.list_ports.return_value = [ironic_port('aaa')]
        client.node.list.return_value = [node1, node2]
        expected = {'mac': {'aaa': 'abcdef'},
                    'pm_addr': {'10.0.1.2': 'fedcba'}}
        self.assertEqual(expected, nodes._populate_node_mapping(client))

    def test_populate_node_mapping_ironic_fake_pxe(self):
        client = mock.MagicMock()
        node = mock.MagicMock()
        node.to_dict.return_value = {'uuid': 'abcdef'}
        ironic_node = collections.namedtuple('node', ['uuid', 'driver',
                                             'driver_info'])
        ironic_port = collections.namedtuple('port', ['address'])
        node_detail = ironic_node('abcdef', 'fake_pxe', None)
        client.node.get.return_value = node_detail
        client.node.list_ports.return_value = [ironic_port('aaa')]
        client.node.list.return_value = [node]
        expected = {'mac': {'aaa': 'abcdef'}, 'pm_addr': {}}
        self.assertEqual(expected, nodes._populate_node_mapping(client))

    def test_clean_up_extra_nodes_ironic(self):
        node = collections.namedtuple('node', ['uuid'])
        client = mock.MagicMock()
        client.node.list.return_value = [node('foobar')]
        nodes._clean_up_extra_nodes(set(('abcd',)), client, remove=True)
        client.node.delete.assert_called_once_with('foobar')

    def test__get_node_id_fake_pxe(self):
        node = self._get_node()
        node['pm_type'] = 'fake_pxe'
        node_map = {'mac': {'aaa': 'abcdef'}, 'pm_addr': {}}
        self.assertEqual('abcdef', nodes._get_node_id(node, node_map))



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import time

from ironicclient import exc as ironicexp
import six

from os_cloud_config.cmd.utils import _clients as clients
from os_cloud_config import glance

LOG = logging.getLogger(__name__)


# This module is no longer used by TripleO! If you feel like changing one of
# the functions below or adding a new one, please apply your change to
# tripleo_common.utils.nodes in the tripleo-common repo.


def _ipmi_driver_info(node):
    driver_info = {"ipmi_address": node["pm_addr"],
                   "ipmi_username": node["pm_user"],
                   "ipmi_password": node["pm_password"]}
    for params in ('ipmi_bridging', 'ipmi_transit_address',
                   'ipmi_transit_channel', 'ipmi_target_address',
                   'ipmi_target_channel', 'ipmi_local_address'):
        if node.get(params):
            driver_info[params] = node[params]
    return driver_info


def _pxe_drac_driver_info(node):
    driver_info = {"drac_host": node["pm_addr"],
                   "drac_username": node["pm_user"],
                   "drac_password": node["pm_password"]}
    return driver_info


def _pxe_ssh_driver_info(node):
    if "pm_virt_type" not in node:
        node["pm_virt_type"] = "virsh"
    driver_info = {"ssh_address": node["pm_addr"],
                   "ssh_username": node["pm_user"],
                   "ssh_key_contents": node["pm_password"],
                   "ssh_virt_type": node["pm_virt_type"]}
    return driver_info


def _pxe_ilo_driver_info(node):
    driver_info = {"ilo_address": node["pm_addr"],
                   "ilo_username": node["pm_user"],
                   "ilo_password": node["pm_password"]}
    return driver_info


def _pxe_iboot_driver_info(node):
    driver_info = {"iboot_address": node["pm_addr"],
                   "iboot_username": node["pm_user"],
                   "iboot_password": node["pm_password"]}
    # iboot_relay_id and iboot_port are optional
    if "pm_relay_id" in node:
        driver_info["iboot_relay_id"] = node["pm_relay_id"]
    if "pm_port" in node:
        driver_info["iboot_port"] = node["pm_port"]
    return driver_info


def _fake_pxe_driver_info(node):
    driver_info = {}
    # The fake_pxe driver doesn't need any credentials since there's
    # no power management
    return driver_info


def _pxe_ucs_driver_info(node):
    driver_info = {"ucs_address": node["pm_addr"],
                   "ucs_username": node["pm_user"],
                   "ucs_password": node["pm_password"],
                   "ucs_service_profile": node["pm_service_profile"]}
    return driver_info


def _common_irmc_driver_info(node):
    driver_info = {"irmc_address": node["pm_addr"],
                   "irmc_username": node["pm_user"],
                   "irmc_password": node["pm_password"]}
    # irmc_port, irmc_auth_method, irmc_client_timeout, and
    # irmc_sensor_method are optional
    if "pm_port" in node:
        driver_info["irmc_port"] = node["pm_port"]
    if "pm_auth_method" in node:
        driver_info["irmc_auth_method"] = node["pm_auth_method"]
    if "pm_client_timeout" in node:
        driver_info["irmc_client_timeout"] = node["pm_client_timeout"]
    if "pm_sensor_method" in node:
        driver_info["irmc_sensor_method"] = node["pm_sensor_method"]
    return driver_info


def _pxe_irmc_driver_info(node):
    return _common_irmc_driver_info(node)


def _iscsi_irmc_driver_info(node):
    driver_info = _common_irmc_driver_info(node)
    # irmc_deploy_iso is also required for iscsi_irmc
    driver_info["irmc_deploy_iso"] = node["pm_deploy_iso"]
    return driver_info


def _pxe_wol_driver_info(node):
    driver_info = {"wol_host": node["pm_addr"]}
    if "pm_port" in node:
        driver_info["wol_port"] = node["pm_port"]
    return driver_info


def _associate_deploy_kr_info(driver_info, node):
    if "pxe" in node["pm_type"]:
        if "kernel_id" in node:
            driver_info["deploy_kernel"] = node["kernel_id"]
        if "ramdisk_id" in node:
            driver_info["deploy_ramdisk"] = node["ramdisk_id"]
    return driver_info


def _extract_driver_info(node):
    driver_info = {}
    driver_info_map = {"pxe_drac": _pxe_drac_driver_info,
                       "pxe_ssh": _pxe_ssh_driver_info,
                       "pxe_ilo": _pxe_ilo_driver_info,
                       "pxe_iboot_iscsi": _pxe_iboot_driver_info,
                       "pxe_iboot_agent": _pxe_iboot_driver_info,
                       "fake_pxe": _fake_pxe_driver_info,
                       "pxe_ucs": _pxe_ucs_driver_info,
                       "pxe_irmc": _pxe_irmc_driver_info,
                       "iscsi_irmc": _iscsi_irmc_driver_info,
                       # agent_irmc and iscsi_irmc share the same driver info
                       "agent_irmc": _iscsi_irmc_driver_info,
                       "pxe_wol": _pxe_wol_driver_info}

    def _get_driver_info(node):
        pm_type = node["pm_type"]
        if "ipmi" in pm_type:
            return _ipmi_driver_info(node)
        else:
            if pm_type in driver_info_map:
                return driver_info_map[pm_type](node)
            else:
                raise ValueError("Unknown pm_type: %s" % node["pm_type"])

    driver_info = _get_driver_info(node)
    driver_info = _associate_deploy_kr_info(driver_info, node)
    return driver_info


def register_ironic_node(service_host, node, client=None, blocking=True):
    mapping = {'cpus': 'cpu',
               'memory_mb': 'memory',
               'local_gb': 'disk',
               'cpu_arch': 'arch'}
    properties = {k: six.text_type(node.get(v))
                  for k, v in mapping.items()
                  if node.get(v) is not None}
    driver_info = _extract_driver_info(node)

    if 'capabilities' in node:
        properties.update({"capabilities":
                          six.text_type(node.get('capabilities'))})

    create_map = {"driver": node["pm_type"],
                  "properties": properties,
                  "driver_info": driver_info}

    if 'name' in node:
        create_map.update({"name": six.text_type(node.get('name'))})

    for count in range(60):
        LOG.debug('Registering %s node with ironic, try #%d.' %
                  (node.get("pm_addr", ''), count))
        try:
            ironic_node = client.node.create(**create_map)
            break
        except (ironicexp.ConnectionRefused, ironicexp.ServiceUnavailable):
            if blocking:
                LOG.debug('Service not available, sleeping for 10 seconds.')
                time.sleep(10)
            else:
                LOG.debug('Service not available.')
    else:
        if blocking:
            LOG.debug('Service unavailable after 10 minutes, giving up.')
        else:
            LOG.debug('Service unavailable after 60 tries, giving up.')
        raise ironicexp.ServiceUnavailable()

    for mac in node["mac"]:
        client.port.create(address=mac, node_uuid=ironic_node.uuid)
    # Ironic should do this directly, see bug 1315225.
    try:
        client.node.set_power_state(ironic_node.uuid, 'off')
    except ironicexp.Conflict:
        # Conflict means the Ironic conductor got there first, so we can
        # ignore the exception.
        pass
    return ironic_node


def _populate_node_mapping(client):
    LOG.debug('Populating list of registered nodes.')
    node_map = {'mac': {}, 'pm_addr': {}}
    nodes = [n.to_dict() for n in client.node.list()]
    for node in nodes:
        node_details = client.node.get(node['uuid'])
        if node_details.driver in ('pxe_ssh', 'fake_pxe'):
            for port in client.node.list_ports(node['uuid']):
                node_map['mac'][port.address] = node['uuid']
        elif 'ipmi' in node_details.driver:
            pm_addr = node_details.driver_info['ipmi_address']
            node_map['pm_addr'][pm_addr] = node['uuid']
        elif node_details.driver == 'pxe_ilo':
            pm_addr = node_details.driver_info['ilo_address']
            node_map['pm_addr'][pm_addr] = node['uuid']
        elif node_details.driver == 'pxe_drac':
            pm_addr = node_details.driver_info['drac_host']
            node_map['pm_addr'][pm_addr] = node['uuid']
        elif node_details.driver in ('pxe_iboot_iscsi', 'pxe_iboot_agent'):
            iboot_addr = node_details.driver_info['iboot_address']
            if "iboot_port" in node_details.driver_info:
                iboot_addr += (':%s' %
                               node_details.driver_info['iboot_port'])
            if "iboot_relay_id" in node_details.driver_info:
                iboot_addr += ('#%s' %
                               node_details.driver_info['iboot_relay_id'])
            node_map['pm_addr'][iboot_addr] = node['uuid']
        elif node_details.driver == 'pxe_irmc':
            pm_addr = node_details.driver_info['irmc_address']
            node_map['pm_addr'][pm_addr] = node['uuid']
    return node_map


def _get_node_id(node, node_map):
    if node['pm_type'] in ('pxe_ssh', 'fake_pxe'):
        for mac in node['mac']:
            if mac.lower() in node_map['mac']:
                return node_map['mac'][mac.lower()]
    elif node['pm_type'] in ('pxe_iboot_iscsi', 'pxe_iboot_agent'):
        iboot_addr = node["pm_addr"]
        if "pm_port" in node:
            iboot_addr += ':%s' % node["pm_port"]
        if "pm_relay_id" in node:
            iboot_addr += '#%s' % node["pm_relay_id"]
        if iboot_addr in node_map['pm_addr']:
            return node_map['pm_addr'][iboot_addr]
    else:
        if node['pm_addr'] in node_map['pm_addr']:
            return node_map['pm_addr'][node['pm_addr']]


def _update_or_register_ironic_node(service_host, node, node_map, client=None,
                                    blocking=True):
    node_uuid = _get_node_id(node, node_map)
    massage_map = {'cpu': '/properties/cpus',
                   'memory': '/properties/memory_mb',
                   'disk': '/properties/local_gb',
                   'arch': '/properties/cpu_arch'}
    if "ipmi" in node['pm_type']:
        massage_map.update({'pm_addr': '/driver_info/ipmi_address',
                            'pm_user': '/driver_info/ipmi_username',
                            'pm_password': '/driver_info/ipmi_password'})
    elif node['pm_type'] == 'pxe_ssh':
        massage_map.update({'pm_addr': '/driver_info/ssh_address',
                            'pm_user': '/driver_info/ssh_username',
                            'pm_password': '/driver_info/ssh_key_contents'})
    elif node['pm_type'] == 'pxe_ilo':
        massage_map.update({'pm_addr': '/driver_info/ilo_address',
                            'pm_user': '/driver_info/ilo_username',
                            'pm_password': '/driver_info/ilo_password'})
    elif node['pm_type'] == 'pxe_drac':
        massage_map.update({'pm_addr': '/driver_info/drac_host',
                            'pm_user': '/driver_info/drac_username',
                            'pm_password': '/driver_info/drac_password'})
    elif node['pm_type'] == 'pxe_irmc':
        massage_map.update({'pm_addr': '/driver_info/irmc_address',
                            'pm_user': '/driver_info/irmc_username',
                            'pm_password': '/driver_info/irmc_password'})

    if "name" in node:
        massage_map.update({'name': '/name'})

    if "capabilities" in node:
        massage_map.update({'capabilities': '/properties/capabilities'})

    if node_uuid:
        ironic_node = client.node.get(node_uuid)
    else:
        ironic_node = None
    if ironic_node is None:
        ironic_node = register_ironic_node(service_host, node, client,
                                           blocking=blocking)
    else:
        LOG.debug('Node %s already registered, updating details.' % (
            ironic_node.uuid))
        node_patch = []
        for key, value in massage_map.items():
            node_patch.append({'path': value,
                               'value': six.text_type(node[key]),
                               'op': 'replace'})
        for count in range(2):
            try:
                client.node.update(ironic_node.uuid, node_patch)
                break
            except ironicexp.Conflict:
                LOG.debug('Node %s locked for updating.' %
                          ironic_node.uuid)
                time.sleep(5)
        else:
            raise ironicexp.Conflict()
    return ironic_node.uuid


def _clean_up_extra_nodes(seen, client, remove=False):
    all_nodes = set([n.uuid for n in client.node.list()])
    remove_func = client.node.delete
    extra_nodes = all_nodes - seen
    for node in extra_nodes:
        if remove:
            LOG.debug('Removing extra registered node %s.' % node)
            remove_func(node)
        else:
            LOG.debug('Extra registered node %s found.' % node)


def _register_list_of_nodes(register_func, node_map, client, nodes_list,
                            blocking, service_host, kernel_id, ramdisk_id):
    seen = set()
    for node in nodes_list:
        if kernel_id:
            if 'kernel_id' not in node:
                node['kernel_id'] = kernel_id
        if ramdisk_id:
            if 'ramdisk_id' not in node:
                node['ramdisk_id'] = ramdisk_id
        try:
            new_node = register_func(service_host, node, node_map,
                                     client=client, blocking=blocking)
            seen.add(new_node)
        except ironicexp.Conflict:
            LOG.debug("Could not update node, moving to next host")
            seen.add(node)
    return seen


def register_all_nodes(service_host, nodes_list, client=None, remove=False,
                       blocking=True, keystone_client=None, glance_client=None,
                       kernel_name=None, ramdisk_name=None):
    LOG.warning('Using register_all_nodes from os-cloud-config is deprecated, '
                'please use the same function from tripleo_common.utils.nodes')

    LOG.debug('Registering all nodes.')
    if client is None:
        LOG.warning('Creating ironic client inline is deprecated, please '
                    'pass the client as parameter.')
        client = clients.get_ironic_client()
    register_func = _update_or_register_ironic_node
    node_map = _populate_node_mapping(client)
    glance_ids = {'kernel': None, 'ramdisk': None}
    if kernel_name and ramdisk_name:
        if glance_client is None:
            LOG.warning('Creating glance client inline is deprecated, please '
                        'pass the client as a parameter.')
            client = clients.get_glance_client()
        glance_ids = glance.create_or_find_kernel_and_ramdisk(
            glance_client, kernel_name, ramdisk_name)
    seen = _register_list_of_nodes(register_func, node_map, client,
                                   nodes_list, blocking, service_host,
                                   glance_ids['kernel'], glance_ids['ramdisk'])
    _clean_up_extra_nodes(seen, client, remove=remove)



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import tempfile

import mock

from os_cloud_config.cmd import setup_flavors
from os_cloud_config.tests import base


class RegisterNodesTest(base.TestCase):

    @mock.patch('os_cloud_config.cmd.utils._clients.get_nova_bm_client')
    @mock.patch('os_cloud_config.flavors.create_flavors_from_nodes')
    @mock.patch.dict('os.environ', {'OS_USERNAME': 'a', 'OS_PASSWORD': 'a',
                     'OS_TENANT_NAME': 'a', 'OS_AUTH_URL': 'a'})
    @mock.patch.object(sys, 'argv', ['setup-flavors', '--nodes', '-k', 'aaa',
                       '-r', 'zzz'])
    def test_with_arguments_nodes(self, create_flavors_mock,
                                  get_nova_bm_client_mock):
        with tempfile.NamedTemporaryFile() as f:
            f.write(u'{}\n'.encode('utf-8'))
            f.flush()
            sys.argv.insert(2, f.name)
            return_code = setup_flavors.main()

        create_flavors_mock.assert_called_once_with(
            get_nova_bm_client_mock(), {}, 'aaa', 'zzz', None)

        self.assertEqual(0, return_code)

    @mock.patch('os_cloud_config.cmd.utils._clients.get_nova_bm_client')
    @mock.patch('os_cloud_config.flavors.create_flavors_from_nodes')
    @mock.patch.dict('os.environ', {'OS_USERNAME': 'a', 'OS_PASSWORD': 'a',
                                    'OS_TENANT_NAME': 'a', 'OS_AUTH_URL': 'a',
                                    'ROOT_DISK': '10'})
    @mock.patch.object(sys, 'argv', ['setup-flavors', '--nodes', '-k', 'aaa',
                       '-r', 'zzz'])
    def test_with_arguments_nodes_root_disk(self, create_flavors_mock,
                                            get_nova_bm_client_mock):
        with tempfile.NamedTemporaryFile() as f:
            f.write(u'{}\n'.encode('utf-8'))
            f.flush()
            sys.argv.insert(2, f.name)
            return_code = setup_flavors.main()

        create_flavors_mock.assert_called_once_with(
            get_nova_bm_client_mock(), {}, 'aaa', 'zzz', '10')

        self.assertEqual(0, return_code)

    @mock.patch('os_cloud_config.cmd.utils._clients.get_nova_bm_client')
    @mock.patch('os_cloud_config.flavors.create_flavors_from_list')
    @mock.patch.dict('os.environ', {'OS_USERNAME': 'a', 'OS_PASSWORD': 'a',
                     'OS_TENANT_NAME': 'a', 'OS_AUTH_URL': 'a'})
    @mock.patch.object(sys, 'argv', ['setup-flavors', '--flavors', '-k', 'aaa',
                       '-r', 'zzz'])
    def test_with_arguments_flavors(self, create_flavors_mock,
                                    get_nova_bm_client_mock):
        with tempfile.NamedTemporaryFile() as f:
            f.write(u'{}\n'.encode('utf-8'))
            f.flush()
            sys.argv.insert(2, f.name)
            return_code = setup_flavors.main()

        create_flavors_mock.assert_called_once_with(
            get_nova_bm_client_mock(), {}, 'aaa', 'zzz')

        self.assertEqual(0, return_code)

    @mock.patch('os_cloud_config.cmd.utils._clients.get_nova_bm_client',
                return_value='nova_bm_client_mock')
    @mock.patch('os_cloud_config.flavors.create_flavors_from_nodes')
    @mock.patch.dict('os.environ', {'OS_USERNAME': 'a', 'OS_PASSWORD': 'a',
                     'OS_TENANT_NAME': 'a', 'OS_AUTH_URL': 'a'})
    @mock.patch.object(sys, 'argv', ['setup-flavors', '--nodes', '-k', 'aaa',
                       '-r', 'zzz'])
    def test_with_exception(self, create_flavors_mock,
                            get_nova_bm_client_mock):
        create_flavors_mock.side_effect = ValueError
        with tempfile.NamedTemporaryFile() as f:
            f.write(u'{}\n'.encode('utf-8'))
            f.flush()
            sys.argv.insert(2, f.name)
            return_code = setup_flavors.main()
        self.assertEqual(1, return_code)



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneclient import exceptions
import keystoneclient.v2_0.client as ksclient_v2
import mock

from os_cloud_config import keystone
from os_cloud_config.tests import base


class KeystoneTest(base.TestCase):

    def assert_endpoint(self, host, region='regionOne', public_endpoint=None,
                        admin_endpoint=None, internal_endpoint=None):
        self.client.services.create.assert_called_once_with(
            'keystone', 'identity', description='Keystone Identity Service')
        if public_endpoint is None:
            public_endpoint = 'http://%s:5000/v2.0' % host
        if admin_endpoint is None:
            admin_endpoint = 'http://%s:35357/v2.0' % host
        if internal_endpoint is None:
            internal_endpoint = 'http://%s:5000/v2.0' % host
        self.client.endpoints.create.assert_called_once_with(
            region, self.client.services.create.return_value.id,
            public_endpoint, admin_endpoint, internal_endpoint)

    def assert_calls_in_grant_admin_user_roles(self):
        self.client_v3.roles.list.assert_has_calls([mock.call(name='admin')])
        self.client_v3.domains.list.assert_called_once_with(name='default')
        self.client_v3.users.list.assert_called_once_with(
            domain=self.client_v3.domains.list.return_value[0], name='admin')
        self.client_v3.projects.list.assert_called_once_with(
            domain=self.client_v3.domains.list.return_value[0], name='admin')

    @mock.patch('subprocess.check_call')
    def test_initialize(self, check_call_mock):
        self._patch_client()
        self._patch_client_v3()

        self.client.services.findall.return_value = []
        self.client.endpoints.findall.return_value = []
        self.client.roles.findall.return_value = []
        self.client.tenants.findall.return_value = []

        keystone.initialize(
            '192.0.0.3', 'mytoken', 'admin@example.org', 'adminpasswd')

        self.client.tenants.create.assert_has_calls(
            [mock.call('admin', None), mock.call('service', None)])

        self.assert_calls_in_grant_admin_user_roles()

        self.assert_endpoint('192.0.0.3')

        check_call_mock.assert_called_once_with(
            ["ssh", "-o" "StrictHostKeyChecking=no", "-t", "-l", "root",
             "192.0.0.3", "sudo", "keystone-manage", "pki_setup",
             "--keystone-user",
             "$(getent passwd | grep '^keystone' | cut -d: -f1)",
             "--keystone-group",
             "$(getent group | grep '^keystone' | cut -d: -f1)"])

    def test_initialize_for_swift(self):
        self._patch_client()

        keystone.initialize_for_swift('192.0.0.3', 'mytoken')

        self.client.roles.create.assert_has_calls(
            [mock.call('swiftoperator'), mock.call('ResellerAdmin')])

    def test_initialize_for_heat(self):
        client = mock.MagicMock()
        client.domains.find.side_effect = exceptions.NotFound
        client.users.find.side_effect = exceptions.NotFound

        keystone.initialize_for_heat(client, 'heatadminpasswd')

        client.domains.create.assert_called_once_with(
            'heat', description='Owns users and tenants created by heat')
        client.users.create.assert_called_once_with(
            'heat_domain_admin',
            description='Manages users and tenants created by heat',
            domain=client.domains.create.return_value,
            password='heatadminpasswd')
        client.roles.find.assert_called_once_with(name='admin')
        client.roles.grant.assert_called_once_with(
            client.roles.find.return_value,
            user=client.users.create.return_value,
            domain=client.domains.create.return_value)

    @mock.patch('subprocess.check_call')
    def test_idempotent_initialize(self, check_call_mock):
        self._patch_client()
        self._patch_client_v3()

        self.client.services.findall.return_value = mock.MagicMock()
        self.client.endpoints.findall.return_value = mock.MagicMock()
        self.client.roles.findall.return_value = mock.MagicMock()
        self.client.tenants.findall.return_value = mock.MagicMock()

        keystone.initialize(
            '192.0.0.3',
            'mytoken',
            'admin@example.org',
            'adminpasswd')

        self.assertFalse(self.client.roles.create('admin').called)
        self.assertFalse(self.client.roles.create('service').called)

        self.assertFalse(self.client.tenants.create('admin', None).called)
        self.assertFalse(self.client.tenants.create('service', None).called)

        self.assert_calls_in_grant_admin_user_roles()

        check_call_mock.assert_called_once_with(
            ["ssh", "-o" "StrictHostKeyChecking=no", "-t", "-l", "root",
             "192.0.0.3", "sudo", "keystone-manage", "pki_setup",
             "--keystone-user",
             "$(getent passwd | grep '^keystone' | cut -d: -f1)",
             "--keystone-group",
             "$(getent group | grep '^keystone' | cut -d: -f1)"])

    def test_setup_roles(self):
        self._patch_client()

        self.client.roles.findall.return_value = []

        keystone._setup_roles(self.client)

        self.client.roles.findall.assert_has_calls(
            [mock.call(name='swiftoperator'), mock.call(name='ResellerAdmin'),
             mock.call(name='heat_stack_user')])

        self.client.roles.create.assert_has_calls(
            [mock.call('swiftoperator'), mock.call('ResellerAdmin'),
             mock.call('heat_stack_user')])

    def test_idempotent_setup_roles(self):
        self._patch_client()

        self.client.roles.findall.return_value = mock.MagicMock()

        keystone._setup_roles(self.client)

        self.client.roles.findall.assert_has_calls(
            [mock.call(name='swiftoperator'), mock.call(name='ResellerAdmin'),
             mock.call(name='heat_stack_user')], any_order=True)

        self.assertFalse(self.client.roles.create('swiftoperator').called)
        self.assertFalse(self.client.roles.create('ResellerAdmin').called)
        self.assertFalse(self.client.roles.create('heat_stack_user').called)

    def test_create_tenants(self):
        self._patch_client()

        self.client.tenants.findall.return_value = []

        keystone._create_tenants(self.client)

        self.client.tenants.findall.assert_has_calls(
            [mock.call(name='admin'), mock.call(name='service')],
            any_order=True)

        self.client.tenants.create.assert_has_calls(
            [mock.call('admin', None), mock.call('service', None)])

    def test_idempotent_create_tenants(self):
        self._patch_client()

        self.client.tenants.findall.return_value = mock.MagicMock()

        keystone._create_tenants(self.client)

        self.client.tenants.findall.assert_has_calls(
            [mock.call(name='admin'), mock.call(name='service')],
            any_order=True)

        # Test that tenants are not created again if they exists
        self.assertFalse(self.client.tenants.create('admin', None).called)
        self.assertFalse(self.client.tenants.create('service', None).called)

    def test_create_keystone_endpoint_ssl(self):
        self._patch_client()

        self.client.services.findall.return_value = []
        self.client.endpoints.findall.return_value = []

        keystone._create_keystone_endpoint(
            self.client, '192.0.0.3', 'regionOne', 'keystone.example.com',
            None, None, None)
        public_endpoint = 'https://keystone.example.com:13000/v2.0'
        self.assert_endpoint('192.0.0.3', public_endpoint=public_endpoint)

    def test_create_keystone_endpoint_public(self):
        self._patch_client()

        self.client.services.findall.return_value = []
        self.client.endpoints.findall.return_value = []

        keystone._create_keystone_endpoint(
            self.client, '192.0.0.3', 'regionOne', None, 'keystone.public',
            None, None)
        public_endpoint = 'http://keystone.public:5000/v2.0'
        self.assert_endpoint('192.0.0.3', public_endpoint=public_endpoint)

    def test_create_keystone_endpoint_ssl_and_public(self):
        self._patch_client()

        self.client.services.findall.return_value = []
        self.client.endpoints.findall.return_value = []

        keystone._create_keystone_endpoint(
            self.client, '192.0.0.3', 'regionOne', 'keystone.example.com',
            'keystone.public', None, None)
        public_endpoint = 'https://keystone.example.com:13000/v2.0'
        self.assert_endpoint('192.0.0.3', public_endpoint=public_endpoint)

    def test_create_keystone_endpoint_public_and_admin(self):
        self._patch_client()

        self.client.services.findall.return_value = []
        self.client.endpoints.findall.return_value = []

        keystone._create_keystone_endpoint(
            self.client, '192.0.0.3', 'regionOne', None, 'keystone.public',
            'keystone.admin', None)
        public_endpoint = 'http://keystone.public:5000/v2.0'
        admin_endpoint = 'http://keystone.admin:35357/v2.0'
        self.assert_endpoint('192.0.0.3', public_endpoint=public_endpoint,
                             admin_endpoint=admin_endpoint)

    def test_create_keystone_endpoint_ssl_public_and_admin(self):
        self._patch_client()

        self.client.services.findall.return_value = []
        self.client.endpoints.findall.return_value = []

        keystone._create_keystone_endpoint(
            self.client, '192.0.0.3', 'regionOne', 'keystone.example.com',
            'keystone.public', 'keystone.admin', None)
        public_endpoint = 'https://keystone.example.com:13000/v2.0'
        admin_endpoint = 'http://keystone.admin:35357/v2.0'
        self.assert_endpoint('192.0.0.3', public_endpoint=public_endpoint,
                             admin_endpoint=admin_endpoint)

    def test_create_keystone_endpoint_public_admin_and_internal(self):
        self._patch_client()

        self.client.services.findall.return_value = []
        self.client.endpoints.findall.return_value = []

        keystone._create_keystone_endpoint(
            self.client, '192.0.0.3', 'regionOne', None, 'keystone.public',
            'keystone.admin', 'keystone.internal')
        public_endpoint = 'http://keystone.public:5000/v2.0'
        admin_endpoint = 'http://keystone.admin:35357/v2.0'
        internal_endpoint = 'http://keystone.internal:5000/v2.0'
        self.assert_endpoint('192.0.0.3', public_endpoint=public_endpoint,
                             admin_endpoint=admin_endpoint,
                             internal_endpoint=internal_endpoint)

    def test_create_keystone_endpoint_ssl_public_admin_and_internal(self):
        self._patch_client()

        self.client.services.findall.return_value = []
        self.client.endpoints.findall.return_value = []

        keystone._create_keystone_endpoint(
            self.client, '192.0.0.3', 'regionOne', 'keystone.example.com',
            'keystone.public', 'keystone.admin', 'keystone.internal')
        public_endpoint = 'https://keystone.example.com:13000/v2.0'
        admin_endpoint = 'http://keystone.admin:35357/v2.0'
        internal_endpoint = 'http://keystone.internal:5000/v2.0'
        self.assert_endpoint('192.0.0.3', public_endpoint=public_endpoint,
                             admin_endpoint=admin_endpoint,
                             internal_endpoint=internal_endpoint)

    def test_create_keystone_endpoint_region(self):
        self._patch_client()

        self.client.services.findall.return_value = []
        self.client.endpoints.findall.return_value = []

        keystone._create_keystone_endpoint(
            self.client, '192.0.0.3', 'regionTwo', None, None, None, None)
        self.assert_endpoint('192.0.0.3', region='regionTwo')

    def test_create_keystone_endpoint_ipv6(self):
        self._patch_client()

        self.client.services.findall.return_value = []
        self.client.endpoints.findall.return_value = []

        keystone._create_keystone_endpoint(
            self.client, '2001:db8:fd00:1000:f816:3eff:fec2:8e7c',
            'regionOne',
            None,
            '2001:db8:fd00:1000:f816:3eff:fec2:8e7d',
            '2001:db8:fd00:1000:f816:3eff:fec2:8e7e',
            '2001:db8:fd00:1000:f816:3eff:fec2:8e7f')
        pe = 'http://[2001:db8:fd00:1000:f816:3eff:fec2:8e7d]:5000/v2.0'
        ae = 'http://[2001:db8:fd00:1000:f816:3eff:fec2:8e7e]:35357/v2.0'
        ie = 'http://[2001:db8:fd00:1000:f816:3eff:fec2:8e7f]:5000/v2.0'
        self.assert_endpoint(
            '[2001:db8:fd00:1000:f816:3eff:fec2:8e7c]',
            region='regionOne', public_endpoint=pe, admin_endpoint=ae,
            internal_endpoint=ie)

    @mock.patch('time.sleep')
    def test_create_roles_retry(self, sleep):
        self._patch_client()
        side_effect = (exceptions.ConnectionRefused,
                       exceptions.ServiceUnavailable, mock.DEFAULT,
                       mock.DEFAULT)
        self.client.roles.create.side_effect = side_effect
        self.client.roles.findall.return_value = []

        keystone._create_roles(self.client)
        sleep.assert_has_calls([mock.call(10), mock.call(10)])

    def test_setup_endpoints(self):
        self.client = mock.MagicMock()
        self.client.users.find.side_effect = ksclient_v2.exceptions.NotFound()
        self.client.services.findall.return_value = []
        self.client.endpoints.findall.return_value = []

        keystone.setup_endpoints(
            {'nova': {'password': 'pass', 'type': 'compute',
                      'ssl_port': 1234}},
            public_host='192.0.0.4', region='region', client=self.client,
            os_auth_url='https://192.0.0.3')

        self.client.users.find.assert_called_once_with(name='nova')
        self.client.tenants.find.assert_called_once_with(name='service')
        self.client.roles.find.assert_called_once_with(name='admin')
        self.client.services.findall.assert_called_once_with(type='compute')
        self.client.endpoints.findall.assert_called_once_with(
            publicurl='https://192.0.0.4:1234/v2.1/$(tenant_id)s')

        self.client.users.create.assert_called_once_with(
            'nova', 'pass',
            tenant_id=self.client.tenants.find.return_value.id,
            email='email=nobody@example.com')

        self.client.roles.add_user_role.assert_called_once_with(
            self.client.users.create.return_value,
            self.client.roles.find.return_value,
            self.client.tenants.find.return_value)

        self.client.services.create.assert_called_once_with(
            'nova', 'compute', description='Nova Compute Service')
        self.client.endpoints.create.assert_called_once_with(
            'region',
            self.client.services.create.return_value.id,
            'https://192.0.0.4:1234/v2.1/$(tenant_id)s',
            'http://192.0.0.3:8774/v2.1/$(tenant_id)s',
            'http://192.0.0.3:8774/v2.1/$(tenant_id)s')

    def test_setup_endpoints_ipv6(self):
        self.client = mock.MagicMock()
        self.client.users.find.side_effect = ksclient_v2.exceptions.NotFound()
        self.client.services.findall.return_value = []
        self.client.endpoints.findall.return_value = []

        keystone.setup_endpoints(
            {'nova': {'password': 'pass', 'type': 'compute',
                      'ssl_port': 1234}},
            public_host='2001:db8:fd00:1000:f816:3eff:fec2:8e7c',
            region='region', client=self.client,
            os_auth_url='https://[2001:db8:fd00:1000:f816:3eff:fec2:8e7c]')

        self.client.users.find.assert_called_once_with(name='nova')
        self.client.tenants.find.assert_called_once_with(name='service')
        self.client.roles.find.assert_called_once_with(name='admin')
        self.client.services.findall.assert_called_once_with(type='compute')
        self.client.endpoints.findall.assert_called_once_with(
            publicurl='https://[2001:db8:fd00:1000:f816:3eff:fec2:8e7c]'
                      ':1234/v2.1/$(tenant_id)s')

        self.client.users.create.assert_called_once_with(
            'nova', 'pass',
            tenant_id=self.client.tenants.find.return_value.id,
            email='email=nobody@example.com')

        self.client.roles.add_user_role.assert_called_once_with(
            self.client.users.create.return_value,
            self.client.roles.find.return_value,
            self.client.tenants.find.return_value)

        self.client.services.create.assert_called_once_with(
            'nova', 'compute', description='Nova Compute Service')
        ipv6_addr = '2001:db8:fd00:1000:f816:3eff:fec2:8e7c'
        self.client.endpoints.create.assert_called_once_with(
            'region',
            self.client.services.create.return_value.id,
            'https://[%s]:1234/v2.1/$(tenant_id)s' % ipv6_addr,
            'http://[%s]:8774/v2.1/$(tenant_id)s' % ipv6_addr,
            'http://[%s]:8774/v2.1/$(tenant_id)s' % ipv6_addr)

    @mock.patch('os_cloud_config.keystone._create_service')
    def test_create_ssl_endpoint_no_ssl_port(self, mock_create_service):
        client = mock.Mock()
        client.endpoints.findall.return_value = []
        data = {'nouser': True,
                'internal_host': 'internal',
                'public_host': 'public',
                'port': 1234,
                'password': 'password',
                'type': 'compute',
                }
        mock_service = mock.Mock()
        mock_service.id = 1
        mock_create_service.return_value = mock_service
        keystone._register_endpoint(client, 'fake', data)
        client.endpoints.create.assert_called_once_with(
            'regionOne', 1,
            'http://public:1234/',
            'http://internal:1234/',
            'http://internal:1234/')

    def test_idempotent_register_endpoint(self):
        self.client = mock.MagicMock()

        # Explicitly defining that endpoint has been already created
        self.client.users.find.return_value = mock.MagicMock()
        self.client.services.findall.return_value = mock.MagicMock()
        self.client.endpoints.findall.return_value = mock.MagicMock()

        keystone._register_endpoint(
            self.client,
            'nova',
            {'password': 'pass', 'type': 'compute',
             'ssl_port': 1234, 'public_host': '192.0.0.4',
             'internal_host': '192.0.0.3'},
            region=None)

        # Calling just a subset of find APIs
        self.client.users.find.assert_called_once_with(name='nova')
        self.assertFalse(self.client.tenants.find.called)
        self.assertFalse(self.client.roles.find.called)
        self.client.services.findall.assert_called_once_with(type='compute')
        self.client.endpoints.findall.assert_called_once_with(
            publicurl='https://192.0.0.4:1234/')

        # None of creating API calls has been called
        self.assertFalse(self.client.users.create.called)
        self.assertFalse(self.client.roles.add_user_role.called)
        self.assertFalse(self.client.services.create.called)
        self.assertFalse(self.client.endpoints.create.called)

    @mock.patch('os_cloud_config.keystone.ksclient_v2.Client')
    def test_create_admin_client_v2(self, client):
        self.assertEqual(
            client.return_value,
            keystone._create_admin_client_v2('192.0.0.3', 'mytoken'))
        client.assert_called_once_with(endpoint='http://192.0.0.3:35357/v2.0',
                                       token='mytoken')

    def _patch_client(self):
        self.client = mock.MagicMock()
        self.create_admin_client_patcher = mock.patch(
            'os_cloud_config.keystone._create_admin_client_v2')
        create_admin_client = self.create_admin_client_patcher.start()
        self.addCleanup(self._patch_client_cleanup)
        create_admin_client.return_value = self.client

    def _patch_client_cleanup(self):
        self.create_admin_client_patcher.stop()
        self.client = None

    @mock.patch('os_cloud_config.keystone.ksclient_v3.Client')
    def test_create_admin_client_v3(self, client_v3):
        self.assertEqual(
            client_v3.return_value,
            keystone._create_admin_client_v3('192.0.0.3', 'mytoken'))
        client_v3.assert_called_once_with(endpoint='http://192.0.0.3:35357/v3',
                                          token='mytoken')

    def _patch_client_v3(self):
        self.client_v3 = mock.MagicMock()
        self.create_admin_client_patcher_v3 = mock.patch(
            'os_cloud_config.keystone._create_admin_client_v3')
        create_admin_client_v3 = self.create_admin_client_patcher_v3.start()
        self.addCleanup(self._patch_client_cleanup_v3)
        create_admin_client_v3.return_value = self.client_v3

    def _patch_client_cleanup_v3(self):
        self.create_admin_client_patcher_v3.stop()
        self.client_v3 = None

    def test_create_admin_user_user_exists(self):
        self._patch_client()
        keystone._create_admin_user(self.client, 'admin@example.org',
                                    'adminpasswd')
        self.client.tenants.find.assert_called_once_with(name='admin')
        self.client.users.create.assert_not_called()

    def test_create_admin_user_user_does_not_exist(self):
        self._patch_client()
        self.client.users.find.side_effect = exceptions.NotFound()
        keystone._create_admin_user(self.client, 'admin@example.org',
                                    'adminpasswd')
        self.client.tenants.find.assert_called_once_with(name='admin')
        self.client.users.create.assert_called_once_with(
            'admin', email='admin@example.org', password='adminpasswd',
            tenant_id=self.client.tenants.find.return_value.id)

    def test_grant_admin_user_roles_idempotent(self):
        self._patch_client_v3()
        self.client_v3.roles.list.return_value = (
            [self.client_v3.roles.list.return_value['admin']])
        keystone._grant_admin_user_roles(self.client_v3)
        self.assert_calls_in_grant_admin_user_roles()
        self.client_v3.roles.grant.assert_not_called()

    def list_roles_side_effect(self, *args, **kwargs):
        if kwargs.get('name') == 'admin':
            return [self.client_v3.roles.list.return_value[0]]
        else:
            return []

    def test_grant_admin_user_roles(self):
        self._patch_client_v3()
        self.client_v3.roles.list.side_effect = self.list_roles_side_effect
        keystone._grant_admin_user_roles(self.client_v3)
        self.assert_calls_in_grant_admin_user_roles()
        self.client_v3.roles.grant.assert_has_calls([
            mock.call(self.client_v3.roles.list.return_value[0],
                      user=self.client_v3.users.list.return_value[0],
                      project=self.client_v3.projects.list.return_value[0]),
            mock.call(self.client_v3.roles.list.return_value[0],
                      user=self.client_v3.users.list.return_value[0],
                      domain=self.client_v3.domains.list.return_value[0])])



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

import mock

from os_cloud_config.cmd import init_keystone_heat_domain
from os_cloud_config.tests import base


class InitKeystoneHeatDomainTest(base.TestCase):

    @mock.patch('os_cloud_config.cmd.init_keystone_heat_domain.environment')
    @mock.patch('os_cloud_config.cmd.init_keystone_heat_domain'
                '.initialize_for_heat')
    @mock.patch('os_cloud_config.cmd.utils._clients.get_keystone_v3_client',
                return_value='keystone_v3_client_mock')
    @mock.patch.object(sys, 'argv', ['init-keystone', '-d', 'password'])
    def test_script(self, environment_mock, initialize_mock, client_mock):
        init_keystone_heat_domain.main()
        initialize_mock.assert_called_once_with('keystone_v3_client_mock',
                                                'password')



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import tempfile

from glanceclient import exc
import mock
import testtools

from os_cloud_config import glance
from os_cloud_config.tests import base


class GlanceTest(base.TestCase):

    def setUp(self):
        super(GlanceTest, self).setUp()
        self.image = collections.namedtuple('image', ['id'])

    def test_return_existing_kernel_and_ramdisk(self):
        client = mock.MagicMock()
        expected = {'kernel': 'aaa', 'ramdisk': 'zzz'}
        client.images.find.side_effect = (self.image('aaa'), self.image('zzz'))
        ids = glance.create_or_find_kernel_and_ramdisk(client, 'bm-kernel',
                                                       'bm-ramdisk')
        client.images.create.assert_not_called()
        self.assertEqual(expected, ids)

    def test_raise_exception_kernel(self):
        client = mock.MagicMock()
        client.images.find.side_effect = exc.HTTPNotFound
        message = "Kernel image not found in Glance, and no path specified."
        with testtools.ExpectedException(ValueError, message):
            glance.create_or_find_kernel_and_ramdisk(client, 'bm-kernel',
                                                     None)

    def test_raise_exception_ramdisk(self):
        client = mock.MagicMock()
        client.images.find.side_effect = (self.image('aaa'),
                                          exc.HTTPNotFound)
        message = "Ramdisk image not found in Glance, and no path specified."
        with testtools.ExpectedException(ValueError, message):
            glance.create_or_find_kernel_and_ramdisk(client, 'bm-kernel',
                                                     'bm-ramdisk')

    def test_skip_missing_no_kernel(self):
        client = mock.MagicMock()
        client.images.find.side_effect = (exc.HTTPNotFound,
                                          self.image('bbb'))
        expected = {'kernel': None, 'ramdisk': 'bbb'}
        ids = glance.create_or_find_kernel_and_ramdisk(
            client, 'bm-kernel', 'bm-ramdisk', skip_missing=True)
        self.assertEqual(ids, expected)

    def test_skip_missing_no_ramdisk(self):
        client = mock.MagicMock()
        client.images.find.side_effect = (self.image('aaa'),
                                          exc.HTTPNotFound)
        expected = {'kernel': 'aaa', 'ramdisk': None}
        ids = glance.create_or_find_kernel_and_ramdisk(
            client, 'bm-kernel', 'bm-ramdisk', skip_missing=True)
        self.assertEqual(ids, expected)

    def test_skip_missing_kernel_and_ramdisk(self):
        client = mock.MagicMock()
        client.images.find.side_effect = exc.HTTPNotFound
        expected = {'kernel': None, 'ramdisk': None}
        ids = glance.create_or_find_kernel_and_ramdisk(
            client, 'bm-kernel', 'bm-ramdisk', skip_missing=True)
        self.assertEqual(ids, expected)

    def test_create_kernel_and_ramdisk(self):
        client = mock.MagicMock()
        client.images.find.side_effect = exc.HTTPNotFound
        client.images.create.side_effect = (self.image('aaa'),
                                            self.image('zzz'))
        expected = {'kernel': 'aaa', 'ramdisk': 'zzz'}
        with tempfile.NamedTemporaryFile() as imagefile:
            ids = glance.create_or_find_kernel_and_ramdisk(
                client, 'bm-kernel', 'bm-ramdisk', kernel_path=imagefile.name,
                ramdisk_path=imagefile.name)
        kernel_create = mock.call(name='bm-kernel', disk_format='aki',
                                  is_public=True, data=mock.ANY)
        ramdisk_create = mock.call(name='bm-ramdisk', disk_format='ari',
                                   is_public=True, data=mock.ANY)
        client.images.create.assert_has_calls([kernel_create, ramdisk_create])
        self.assertEqual(expected, ids)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import textwrap

from os_cloud_config.cmd.utils import environment
from os_cloud_config import keystone_pki


def parse_args():
    description = textwrap.dedent("""
    Generate 4 files inside <directory> for use with Keystone PKI
    token signing:

    ca_key.pem       - certificate authority key
    ca_cert.pem      - self-signed certificate authority certificate
    signing_key.pem  - key for signing tokens
    signing_cert.pem - certificate for verifying token validity, the
                       certificate itself is verifiable by ca_cert.pem

    Alternatively write generated key/certs into <heatenv> JSON file
    or into an os-apply-config metadata file for use without Heat.

    ca_key.pem doesn't have to (shouldn't) be uploaded to Keystone nodes.
    """)

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument('-d', '--directory', dest='directory',
                       help='directory where keys/certs will be generated')
    group.add_argument('-j', '--heatenv', dest='heatenv',
                       help='write signing key/cert and CA cert into JSON '
                            'Heat environment file, CA key is omitted')
    parser.add_argument('-s', '--seed', action='store_true',
                        help='JSON file for seed machine has different '
                             'structure (for seed machine we update directly '
                             'heat metadata file injected into image). '
                             'Different key/certs names and different '
                             'parent node are used (default: false)')
    environment._add_logging_arguments(parser)
    return parser.parse_args()


def main():
    args = parse_args()
    environment._configure_logging(args)

    if args.heatenv:
        keystone_pki.generate_certs_into_json(args.heatenv, args.seed)
    else:
        keystone_pki.create_and_write_ca_and_signing_pairs(args.directory)



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import stat

import mock
from OpenSSL import crypto

from os_cloud_config import keystone_pki
from os_cloud_config.tests import base


class KeystonePKITest(base.TestCase):

    def test_create_ca_and_signing_pairs(self):
        # use one common test to avoid generating CA pair twice
        # do not mock out pyOpenSSL, test generated keys/certs

        # create CA pair
        ca_key_pem, ca_cert_pem = keystone_pki.create_ca_pair()
        ca_key = crypto.load_privatekey(crypto.FILETYPE_PEM, ca_key_pem)
        ca_cert = crypto.load_certificate(crypto.FILETYPE_PEM, ca_cert_pem)

        # check CA key properties
        self.assertTrue(ca_key.check())
        self.assertEqual(2048, ca_key.bits())

        # check CA cert properties
        self.assertFalse(ca_cert.has_expired())
        self.assertEqual('Keystone CA', ca_cert.get_issuer().CN)
        self.assertEqual('Keystone CA', ca_cert.get_subject().CN)

        # create signing pair
        signing_key_pem, signing_cert_pem = keystone_pki.create_signing_pair(
            ca_key_pem, ca_cert_pem)
        signing_key = crypto.load_privatekey(crypto.FILETYPE_PEM,
                                             signing_key_pem)
        signing_cert = crypto.load_certificate(crypto.FILETYPE_PEM,
                                               signing_cert_pem)

        # check signing key properties
        self.assertTrue(signing_key.check())
        self.assertEqual(2048, signing_key.bits())

        # check signing cert properties
        self.assertFalse(signing_cert.has_expired())
        self.assertEqual('Keystone CA', signing_cert.get_issuer().CN)
        self.assertEqual('Keystone Signing', signing_cert.get_subject().CN)
        # pyOpenSSL currenty cannot verify a cert against a CA cert

    @mock.patch('os_cloud_config.keystone_pki.os.chmod', create=True)
    @mock.patch('os_cloud_config.keystone_pki.os.mkdir', create=True)
    @mock.patch('os_cloud_config.keystone_pki.path.isdir', create=True)
    @mock.patch('os_cloud_config.keystone_pki.create_ca_pair')
    @mock.patch('os_cloud_config.keystone_pki.create_signing_pair')
    @mock.patch('os_cloud_config.keystone_pki.open', create=True)
    def test_create_and_write_ca_and_signing_pairs(
            self, open_, create_signing, create_ca, isdir, mkdir, chmod):
        create_ca.return_value = ('mock_ca_key', 'mock_ca_cert')
        create_signing.return_value = ('mock_signing_key', 'mock_signing_cert')
        isdir.return_value = False
        keystone_pki.create_and_write_ca_and_signing_pairs('/fake_dir')

        mkdir.assert_called_with('/fake_dir')
        chmod.assert_has_calls([
            mock.call('/fake_dir/ca_key.pem',
                      stat.S_IRUSR | stat.S_IWUSR),
            mock.call('/fake_dir/ca_cert.pem',
                      stat.S_IRUSR | stat.S_IWUSR),
            mock.call('/fake_dir/signing_key.pem',
                      stat.S_IRUSR | stat.S_IWUSR),
            mock.call('/fake_dir/signing_cert.pem',
                      stat.S_IRUSR | stat.S_IWUSR),
        ])
        # need any_order param, there are open().__enter__()
        # etc. called in between
        open_.assert_has_calls([
            mock.call('/fake_dir/ca_key.pem', 'w'),
            mock.call('/fake_dir/ca_cert.pem', 'w'),
            mock.call('/fake_dir/signing_key.pem', 'w'),
            mock.call('/fake_dir/signing_cert.pem', 'w'),
        ], any_order=True)
        cert_files = open_.return_value.__enter__.return_value
        cert_files.write.assert_has_calls([
            mock.call('mock_ca_key'),
            mock.call('mock_ca_cert'),
            mock.call('mock_signing_key'),
            mock.call('mock_signing_cert'),
        ])

    @mock.patch('os_cloud_config.keystone_pki.path.isfile', create=True)
    @mock.patch('os_cloud_config.keystone_pki.create_ca_pair')
    @mock.patch('os_cloud_config.keystone_pki.create_signing_pair')
    @mock.patch('os_cloud_config.keystone_pki.open', create=True)
    @mock.patch('os_cloud_config.keystone_pki.json.dump')
    def test_generate_certs_into_json(
        self, mock_json, open_, create_signing, create_ca, isfile):
        create_ca.return_value = ('mock_ca_key', 'mock_ca_cert')
        create_signing.return_value = ('mock_signing_key', 'mock_signing_cert')
        isfile.return_value = False

        keystone_pki.generate_certs_into_json('/jsonfile', False)

        params = mock_json.call_args[0][0]['parameter_defaults']
        self.assertEqual(params['KeystoneCACertificate'], 'mock_ca_cert')
        self.assertEqual(params['KeystoneSigningKey'], 'mock_signing_key')
        self.assertEqual(params['KeystoneSigningCertificate'],
                         'mock_signing_cert')

    @mock.patch('os_cloud_config.keystone_pki.path.isfile', create=True)
    @mock.patch('os_cloud_config.keystone_pki.create_ca_pair')
    @mock.patch('os_cloud_config.keystone_pki.create_signing_pair')
    @mock.patch('os_cloud_config.keystone_pki.open', create=True)
    @mock.patch('os_cloud_config.keystone_pki.json.load')
    @mock.patch('os_cloud_config.keystone_pki.json.dump')
    def test_generate_certs_into_json_with_existing_certs(
        self, mock_json_dump, mock_json_load, open_, create_signing,
        create_ca, isfile):
        create_ca.return_value = ('mock_ca_key', 'mock_ca_cert')
        create_signing.return_value = ('mock_signing_key', 'mock_signing_cert')
        isfile.return_value = True
        mock_json_load.return_value = {
            'parameter_defaults': {
                'KeystoneCACertificate': 'mock_ca_cert',
                'KeystoneSigningKey': 'mock_signing_key',
                'KeystoneSigningCertificate': 'mock_signing_cert'
            }
        }

        keystone_pki.generate_certs_into_json('/jsonfile', False)
        mock_json_dump.assert_not_called()



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging


LOG = logging.getLogger(__name__)


def cleanup_flavors(client, names=('m1.tiny', 'm1.small', 'm1.medium',
                    'm1.large', 'm1.xlarge')):
    LOG.debug('Cleaning up non-baremetal flavors.')
    for flavor in client.flavors.list():
        if flavor.name in names:
            client.flavors.delete(flavor.id)


def check_node_properties(node):
    for node_property in 'memory_mb', 'local_gb', 'cpus', 'cpu_arch':
        if not node.properties.get(node_property):
            LOG.warning('node %s does not have %s set. Not creating flavor'
                        'from node.', node.uuid, node_property)
            return False
    return True


def create_flavors_from_ironic(client, ironic_client, kernel, ramdisk,
                               root_disk):
    node_list = []
    for node in ironic_client.node.list(detail=True):
        if not check_node_properties(node):
            continue
        node_list.append({
            'memory': node.properties['memory_mb'],
            'disk': node.properties['local_gb'],
            'cpu': node.properties['cpus'],
            'arch': node.properties['cpu_arch']})
    create_flavors_from_nodes(client, node_list, kernel, ramdisk, root_disk)


def create_flavors_from_nodes(client, node_list, kernel, ramdisk, root_disk):
    LOG.debug('Populating flavors to create from node list.')
    node_details = set()
    for node in node_list:
        disk = node['disk']
        ephemeral = 0
        if root_disk:
            disk = str(root_disk)
            ephemeral = str(int(node['disk']) - int(root_disk))
        node_details.add((node['memory'], disk, node['cpu'], node['arch'],
                         ephemeral))
    flavor_list = []
    for node in node_details:
        new_flavor = {'memory': node[0], 'disk': node[1], 'cpu': node[2],
                      'arch': node[3], 'ephemeral': node[4]}
        name = 'baremetal_%(memory)s_%(disk)s_%(ephemeral)s_%(cpu)s' % (
            new_flavor)
        new_flavor['name'] = name
        flavor_list.append(new_flavor)
    create_flavors_from_list(client, flavor_list, kernel, ramdisk)


def create_flavors_from_list(client, flavor_list, kernel, ramdisk):
    LOG.debug('Creating flavors from flavors list.')
    for flavor in filter_existing_flavors(client, flavor_list):
        flavor.update({'kernel': kernel, 'ramdisk': ramdisk})
        LOG.debug('Creating %(name)s flavor with memory %(memory)s, '
                  'disk %(disk)s, cpu %(cpu)s, %(arch)s arch.' % flavor)
        _create_flavor(client, flavor)


def filter_existing_flavors(client, flavor_list):
    flavors = client.flavors.list()
    names_to_create = (set([f['name'] for f in flavor_list]) -
                       set([f.name for f in flavors]))
    flavors_to_create = [f for f in flavor_list
                         if f['name'] in names_to_create]
    return flavors_to_create


def _create_flavor(client, flavor_desc):
    flavor = client.flavors.create(flavor_desc['name'], flavor_desc['memory'],
                                   flavor_desc['cpu'], flavor_desc['disk'],
                                   None, ephemeral=flavor_desc['ephemeral'])
    bm_prefix = 'baremetal:deploy'
    flavor_metadata = {'cpu_arch': flavor_desc['arch'],
                       '%s_kernel_id' % bm_prefix: flavor_desc['kernel'],
                       '%s_ramdisk_id' % bm_prefix: flavor_desc['ramdisk']}
    if flavor_desc.get('extra_specs'):
        flavor_metadata.update(flavor_desc['extra_specs'])
    flavor.set_keys(metadata=flavor_metadata)



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from os_cloud_config.cmd.utils import _clients as clients

LOG = logging.getLogger(__name__)


def initialize_neutron(network_desc, neutron_client=None,
                       keystone_client=None):
    if not neutron_client:
        LOG.warning(
            'Creating neutron client inline is deprecated, please pass '
            'the client as parameter.')
        neutron_client = clients.get_neutron_client()
    if not keystone_client:
        LOG.warning(
            'Creating keystone client inline is deprecated, please pass '
            'the client as parameter.')
        keystone_client = clients.get_keystone_client()

    admin_tenant = _get_admin_tenant_id(keystone_client)
    if 'physical' in network_desc:
        network_type = 'physical'
        if not admin_tenant:
            raise ValueError("No admin tenant registered in Keystone")
        if not network_desc['physical']['metadata_server']:
            raise ValueError("metadata_server is required for physical "
                             "networks")
    elif 'float' in network_desc:
        network_type = 'float'
    else:
        raise ValueError("No float or physical network defined.")
    net = _create_net(neutron_client, network_desc, network_type, admin_tenant)
    subnet = _create_subnet(neutron_client, net, network_desc, network_type,
                            admin_tenant)
    if 'external' in network_desc:
        router = _create_router(neutron_client, subnet)
        ext_net = _create_net(neutron_client, network_desc, 'external', None)
        _create_subnet(neutron_client, ext_net, network_desc, 'external', None)
        neutron_client.add_gateway_router(
            router['router']['id'], {'network_id': ext_net['network']['id']})
    LOG.debug("Neutron configured.")


def _get_admin_tenant_id(keystone):
    """Fetch the admin tenant id from Keystone.

    :param keystone: A keystone v2 client.
    """
    LOG.debug("Discovering admin tenant.")
    tenant = keystone.tenants.find(name='admin')
    if tenant:
        return tenant.id


def _create_net(neutron, network_desc, network_type, admin_tenant):
    """Create a new neutron net.

    :param neutron: A neutron v2 client.
    :param network_desc: A network description.
    :param network_type: The type of network to create.
    :param admin_tenant: The admin tenant id in Keystone.
    """
    LOG.debug("Creating %s network." % network_type)
    type_desc = network_desc[network_type]
    network = {'admin_state_up': True,
               'name': type_desc['name']}
    if network_type == 'physical':
        network.update({'tenant_id': admin_tenant,
                        'provider:network_type': 'flat',
                        'provider:physical_network': network['name']})
    elif network_type == 'float':
        network['shared'] = True
    elif network_type == 'external':
        network['router:external'] = True
    if type_desc.get('segmentation_id'):
        vlan_tag = type_desc['segmentation_id']
        physical_network = type_desc['physical_network']
        network.update({'provider:network_type': 'vlan',
                        'provider:segmentation_id': vlan_tag,
                        'provider:physical_network': physical_network})
    return neutron.create_network({'network': network})


def _create_subnet(neutron, net, network_desc, network_type, admin_tenant):
    """Create a new neutron subnet.

    :param neutron: A neutron v2 client.
    :param network_desc: A network description.
    :param network_type: The type of network to create.
    :param admin_tenant: The admin tenant id in Keystone.
    """
    type_desc = network_desc[network_type]
    cidr = type_desc['cidr']
    LOG.debug("Creating %s subnet, with CIDR %s." % (network_type, cidr))
    subnet = {'ip_version': 4, 'network_id': net['network']['id'],
              'cidr': cidr}
    if network_type == 'physical':
        metadata = network_desc['physical']['metadata_server']
        subnet.update({'tenant_id': admin_tenant,
                       'host_routes': [{'destination': '169.254.169.254/32',
                                       'nexthop': metadata}]})
    elif network_type == 'external':
        subnet['enable_dhcp'] = False
    if type_desc.get('gateway'):
        subnet['gateway_ip'] = type_desc['gateway']
    if type_desc.get('extra_routes'):
        routes = type_desc['extra_routes']
        if 'host_routes' not in subnet:
            subnet['host_routes'] = []
        subnet['host_routes'].extend(routes)
    if type_desc.get('nameserver'):
        subnet['dns_nameservers'] = [type_desc['nameserver']]
    elif network_type == 'float':
        subnet['dns_nameservers'] = ['8.8.8.8']
    if 'enable_dhcp' in type_desc:
        subnet['enable_dhcp'] = type_desc['enable_dhcp']
    if (type_desc.get('allocation_start') and
            type_desc.get('allocation_end')):
        allocation_start = type_desc['allocation_start']
        allocation_end = type_desc['allocation_end']
        subnet['allocation_pools'] = [{'start': allocation_start,
                                       'end': allocation_end}]
    return neutron.create_subnet({'subnet': subnet})


def _create_router(neutron, subnet):
    """Create a new neutron router.

    :param neutron: A neutron v2 client.
    :param subnet: The subnet id to route for.
    """
    LOG.debug("Creating router.")
    router = neutron.create_router({'router': {'name': 'default-router'}})
    neutron.add_interface_router(router['router']['id'],
                                 {'subnet_id': subnet['subnet']['id']})
    return router



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections

import mock

from os_cloud_config import neutron
from os_cloud_config.tests import base


class NeutronTest(base.TestCase):

    def test_get_admin_tenant_id(self):
        client = mock.MagicMock()
        neutron._get_admin_tenant_id(client)
        client.tenants.find.assert_called_once_with(name='admin')

    def test_create_net_physical(self):
        client = mock.MagicMock()
        network = {'physical': {'name': 'ctlplane'}}
        neutron._create_net(client, network, 'physical', 'admin_tenant')
        physical_call = {'network': {'tenant_id': 'admin_tenant',
                                     'provider:network_type': 'flat',
                                     'name': 'ctlplane',
                                     'provider:physical_network': 'ctlplane',
                                     'admin_state_up': True}}
        client.create_network.assert_called_once_with(physical_call)

    def test_create_net_physical_vlan_tag(self):
        client = mock.MagicMock()
        network = {'physical': {'name': 'public',
                                'segmentation_id': '123',
                                'physical_network': 'ctlplane'}}
        neutron._create_net(client, network, 'physical', 'admin_tenant')
        physical_call = {'network': {'tenant_id': 'admin_tenant',
                                     'provider:network_type': 'vlan',
                                     'name': 'public',
                                     'provider:physical_network': 'ctlplane',
                                     'provider:segmentation_id': '123',
                                     'admin_state_up': True}}
        client.create_network.assert_called_once_with(physical_call)

    def test_create_net_float(self):
        client = mock.MagicMock()
        network = {'float': {'name': 'default-net'}}
        neutron._create_net(client, network, 'float', None)
        float_call = {'network': {'shared': True,
                                  'name': 'default-net',
                                  'admin_state_up': True}}
        client.create_network.assert_called_once_with(float_call)

    def test_create_net_external(self):
        client = mock.MagicMock()
        network = {'external': {'name': 'ext-net'}}
        neutron._create_net(client, network, 'external', None)
        external_call = {'network': {'router:external': True,
                                     'name': 'ext-net',
                                     'admin_state_up': True}}
        client.create_network.assert_called_once_with(external_call)

    def test_create_subnet_physical(self):
        client = mock.MagicMock()
        net = {'network': {'id': 'abcd'}}
        network = {'physical': {'name': 'ctlplane',
                                'cidr': '10.0.0.0/24',
                                'metadata_server': '10.0.0.1'}}
        neutron._create_subnet(client, net, network, 'physical',
                               'admin_tenant')
        host_routes = [{'nexthop': '10.0.0.1',
                        'destination': '169.254.169.254/32'}]
        physical_call = {'subnet': {'ip_version': 4,
                                    'network_id': 'abcd',
                                    'cidr': '10.0.0.0/24',
                                    'host_routes': host_routes,
                                    'tenant_id': 'admin_tenant'}}
        client.create_subnet.assert_called_once_with(physical_call)

    def test_create_subnet_float(self):
        client = mock.MagicMock()
        net = {'network': {'id': 'abcd'}}
        network = {'float': {'name': 'default-net',
                             'cidr': '172.16.5.0/24'}}
        neutron._create_subnet(client, net, network, 'float', None)
        float_call = {'subnet': {'ip_version': 4,
                                 'network_id': 'abcd',
                                 'cidr': '172.16.5.0/24',
                                 'dns_nameservers': ['8.8.8.8']}}
        client.create_subnet.assert_called_once_with(float_call)

    def test_create_subnet_external(self):
        client = mock.MagicMock()
        net = {'network': {'id': 'abcd'}}
        network = {'external': {'name': 'ext-net',
                                'cidr': '1.2.3.0/24'}}
        neutron._create_subnet(client, net, network, 'external', None)
        external_call = {'subnet': {'ip_version': 4,
                                    'network_id': 'abcd',
                                    'cidr': '1.2.3.0/24',
                                    'enable_dhcp': False}}
        client.create_subnet.assert_called_once_with(external_call)

    def test_create_subnet_with_gateway(self):
        client = mock.MagicMock()
        net = {'network': {'id': 'abcd'}}
        network = {'external': {'name': 'ext-net',
                                'cidr': '1.2.3.0/24',
                                'gateway': '1.2.3.4'}}
        neutron._create_subnet(client, net, network, 'external', None)
        external_call = {'subnet': {'ip_version': 4,
                                    'network_id': 'abcd',
                                    'cidr': '1.2.3.0/24',
                                    'gateway_ip': '1.2.3.4',
                                    'enable_dhcp': False}}
        client.create_subnet.assert_called_once_with(external_call)

    def test_create_subnet_with_allocation_pool(self):
        client = mock.MagicMock()
        net = {'network': {'id': 'abcd'}}
        network = {'float': {'name': 'default-net',
                             'cidr': '172.16.5.0/24',
                             'allocation_start': '172.16.5.25',
                             'allocation_end': '172.16.5.40'}}
        neutron._create_subnet(client, net, network, 'float', None)
        float_call = {'subnet': {'ip_version': 4,
                                 'network_id': 'abcd',
                                 'cidr': '172.16.5.0/24',
                                 'dns_nameservers': ['8.8.8.8'],
                                 'allocation_pools': [{'start': '172.16.5.25',
                                                       'end': '172.16.5.40'}]}}
        client.create_subnet.assert_called_once_with(float_call)

    def test_create_physical_subnet_with_extra_routes(self):
        client = mock.MagicMock()
        net = {'network': {'id': 'abcd'}}
        routes = [{'destination': '2.3.4.0/24', 'nexthop': '172.16.6.253'}]
        network = {'physical': {'name': 'ctlplane',
                                'cidr': '10.0.0.0/24',
                                'metadata_server': '10.0.0.1',
                                'extra_routes': routes}}
        neutron._create_subnet(client, net, network, 'physical',
                               'admin_tenant')
        host_routes = [{'nexthop': '10.0.0.1',
                        'destination': '169.254.169.254/32'}] + routes
        physical_call = {'subnet': {'ip_version': 4,
                                    'network_id': 'abcd',
                                    'cidr': '10.0.0.0/24',
                                    'host_routes': host_routes,
                                    'tenant_id': 'admin_tenant'}}
        client.create_subnet.assert_called_once_with(physical_call)

    def test_create_float_subnet_with_extra_routes(self):
        client = mock.MagicMock()
        net = {'network': {'id': 'abcd'}}
        routes = [{'destination': '2.3.4.0/24', 'nexthop': '172.16.6.253'}]
        network = {'float': {'name': 'default-net',
                             'cidr': '172.16.6.0/24',
                             'extra_routes': routes}}
        neutron._create_subnet(client, net, network, 'float', None)
        float_call = {'subnet': {'ip_version': 4,
                                 'network_id': 'abcd',
                                 'cidr': '172.16.6.0/24',
                                 'dns_nameservers': ['8.8.8.8'],
                                 'host_routes': routes}}
        client.create_subnet.assert_called_once_with(float_call)

    def test_create_subnet_with_nameserver(self):
        client = mock.MagicMock()
        net = {'network': {'id': 'abcd'}}
        network = {'float': {'name': 'default-net',
                             'cidr': '172.16.5.0/24',
                             'nameserver': '172.16.5.254'}}
        neutron._create_subnet(client, net, network, 'float', None)
        float_call = {'subnet': {'ip_version': 4,
                                 'network_id': 'abcd',
                                 'cidr': '172.16.5.0/24',
                                 'dns_nameservers': ['172.16.5.254']}}
        client.create_subnet.assert_called_once_with(float_call)

    def test_create_subnet_with_no_dhcp(self):
        client = mock.MagicMock()
        net = {'network': {'id': 'abcd'}}
        network = {'physical': {'name': 'ctlplane',
                                'cidr': '10.0.0.0/24',
                                'metadata_server': '10.0.0.1',
                                'enable_dhcp': False}}
        neutron._create_subnet(client, net, network, 'physical', 'tenant')
        host_routes = [{'nexthop': '10.0.0.1',
                        'destination': '169.254.169.254/32'}]
        physical_call = {'subnet': {'ip_version': 4,
                                    'enable_dhcp': False,
                                    'network_id': 'abcd',
                                    'cidr': '10.0.0.0/24',
                                    'host_routes': host_routes,
                                    'tenant_id': 'tenant'}}
        client.create_subnet.assert_called_once_with(physical_call)

    @mock.patch('os_cloud_config.cmd.utils._clients.get_neutron_client')
    @mock.patch('os_cloud_config.cmd.utils._clients.get_keystone_client')
    def test_initialize_neutron_physical(self, keystoneclient, neutronclient):
        network_desc = {'physical': {'name': 'ctlplane',
                                     'cidr': '10.0.0.0/24',
                                     'metadata_server': '10.0.0.1'}}
        tenant = collections.namedtuple('tenant', ['id'])
        keystoneclient().tenants.find.return_value = tenant('dead-beef')
        neutron.initialize_neutron(network_desc)
        network_call = {'network': {'tenant_id': 'dead-beef',
                                    'provider:network_type': 'flat',
                                    'name': u'ctlplane',
                                    'provider:physical_network': u'ctlplane',
                                    'admin_state_up': True}}
        host_routes = [{'nexthop': '10.0.0.1',
                        'destination': '169.254.169.254/32'}]
        net_id = neutronclient().create_network.return_value['network']['id']
        subnet_call = {'subnet': {'ip_version': 4,
                                  'network_id': net_id,
                                  'cidr': u'10.0.0.0/24',
                                  'host_routes': host_routes,
                                  'tenant_id': 'dead-beef'}}
        neutronclient().create_network.assert_called_once_with(network_call)
        neutronclient().create_subnet.assert_called_once_with(subnet_call)

    @mock.patch('os_cloud_config.cmd.utils._clients.get_neutron_client')
    @mock.patch('os_cloud_config.cmd.utils._clients.get_keystone_client')
    def test_initialize_neutron_float_and_external(self, keystoneclient,
                                                   neutronclient):
        network_desc = {'float': {'name': 'default-net',
                                  'cidr': '172.16.5.0/24'},
                        'external': {'name': 'ext-net',
                                     'cidr': '1.2.3.0/24'}}
        tenant = collections.namedtuple('tenant', ['id'])
        keystoneclient().tenants.find.return_value = tenant('dead-beef')
        neutron.initialize_neutron(network_desc)
        float_network = {'network': {'shared': True,
                                     'name': 'default-net',
                                     'admin_state_up': True}}
        external_network = {'network': {'router:external': True,
                                        'name': 'ext-net',
                                        'admin_state_up': True}}
        float_subnet = {'subnet': {'ip_version': 4,
                                   'network_id': mock.ANY,
                                   'cidr': '172.16.5.0/24',
                                   'dns_nameservers': ['8.8.8.8']}}
        external_subnet = {'subnet': {'ip_version': 4,
                                      'network_id': mock.ANY,
                                      'cidr': '1.2.3.0/24',
                                      'enable_dhcp': False}}
        router_call = {'router': {'name': 'default-router'}}
        neutronclient().create_network.has_calls([float_network,
                                                  external_network])
        neutronclient().create_subnet.has_calls([float_subnet,
                                                 external_subnet])
        neutronclient().create_router.assert_called_once_with(router_call)
        network = neutronclient().create_network.return_value
        neutronclient().add_gateway_router.assert_called_once_with(
            neutronclient().create_router.return_value['router']['id'],
            {'network_id': network['network']['id']})



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections

import mock

from os_cloud_config import flavors
from os_cloud_config.tests import base


class FlavorsTest(base.TestCase):

    def test_cleanup_flavors(self):
        client = mock.MagicMock()
        to_del = ('m1.tiny', 'm1.small', 'm1.medium', 'm1.large', 'm1.xlarge')
        delete_calls = [mock.call(flavor) for flavor in to_del]
        flavors.cleanup_flavors(client=client)
        client.flavors.delete.has_calls(delete_calls)

    def test_filter_existing_flavors_none(self):
        client = mock.MagicMock()
        client.flavors.list.return_value = []
        flavor_list = [{'name': 'baremetal'}]
        self.assertEqual(flavor_list,
                         flavors.filter_existing_flavors(client, flavor_list))

    def test_filter_existing_flavors_one_existing(self):
        client = mock.MagicMock()
        flavor = collections.namedtuple('flavor', ['name'])
        client.flavors.list.return_value = [flavor('baremetal_1')]
        flavor_list = [{'name': 'baremetal_0'}, {'name': 'baremetal_1'}]
        self.assertEqual([flavor_list[0]],
                         flavors.filter_existing_flavors(client, flavor_list))

    def test_filter_existing_flavors_all_existing(self):
        client = mock.MagicMock()
        flavor = collections.namedtuple('flavor', ['name'])
        client.flavors.list.return_value = [flavor('baremetal_0'),
                                            flavor('baremetal_1')]
        flavor_list = [{'name': 'baremetal_0'}, {'name': 'baremetal_1'}]
        self.assertEqual([],
                         flavors.filter_existing_flavors(client, flavor_list))

    @mock.patch('os_cloud_config.flavors._create_flavor')
    def test_create_flavors_from_nodes(self, create_flavor):
        node = {'cpu': '1', 'memory': '2048', 'disk': '30', 'arch': 'i386'}
        node_list = [node, node]
        client = mock.MagicMock()
        flavors.create_flavors_from_nodes(client, node_list, 'aaa', 'bbb',
                                          '10')
        expected_flavor = node
        expected_flavor.update({'disk': '10', 'ephemeral': '20',
                                'kernel': 'aaa', 'ramdisk': 'bbb',
                                'name': 'baremetal_2048_10_20_1'})
        create_flavor.assert_called_once_with(client, expected_flavor)

    @mock.patch('os_cloud_config.flavors._create_flavor')
    def test_create_flavors_from_list(self, create_flavor):
        flavor_list = [{'name': 'controller', 'cpu': '1', 'memory': '2048',
                        'disk': '30', 'arch': 'amd64'}]
        client = mock.MagicMock()
        flavors.create_flavors_from_list(client, flavor_list, 'aaa', 'bbb')
        create_flavor.assert_called_once_with(
            client, {'disk': '30', 'cpu': '1', 'arch': 'amd64',
                     'kernel': 'aaa', 'ramdisk': 'bbb', 'memory': '2048',
                     'name': 'controller'})

    def test_create_flavor(self):
        flavor = {'cpu': '1', 'memory': '2048', 'disk': '30', 'arch': 'i386',
                  'kernel': 'aaa', 'ramdisk': 'bbb', 'name': 'baremetal',
                  'ephemeral': None}
        client = mock.MagicMock()
        flavors._create_flavor(client, flavor)
        client.flavors.create.assert_called_once_with(
            'baremetal', '2048', '1', '30', None, ephemeral=None)
        metadata = {'cpu_arch': 'i386', 'baremetal:deploy_kernel_id': 'aaa',
                    'baremetal:deploy_ramdisk_id': 'bbb'}
        client.flavors.create.return_value.set_keys.assert_called_once_with(
            metadata=metadata)

    def test_create_flavor_with_extra_spec(self):
        flavor = {'cpu': '1', 'memory': '2048', 'disk': '30', 'arch': 'i386',
                  'kernel': 'aaa', 'ramdisk': 'bbb', 'name': 'baremetal',
                  'ephemeral': None, 'extra_specs': {'key': 'value'}}
        client = mock.MagicMock()
        flavors._create_flavor(client, flavor)
        client.flavors.create.assert_called_once_with(
            'baremetal', '2048', '1', '30', None, ephemeral=None)
        metadata = {'cpu_arch': 'i386', 'baremetal:deploy_kernel_id': 'aaa',
                    'baremetal:deploy_ramdisk_id': 'bbb', 'key': 'value'}
        client.flavors.create.return_value.set_keys.assert_called_once_with(
            metadata=metadata)

    @mock.patch('os_cloud_config.flavors._create_flavor')
    def test_create_flavor_from_ironic(self, create_flavor):
        node = mock.MagicMock()
        node.uuid = 'uuid'
        node.properties = {'cpus': '1', 'memory_mb': '2048', 'local_gb': '30',
                           'cpu_arch': 'i386'}
        client = mock.MagicMock()
        ironic_client = mock.MagicMock()
        ironic_client.node.list.return_value = [node]
        flavors.create_flavors_from_ironic(client, ironic_client, 'aaa', 'bbb',
                                           '10')
        self.assertTrue(ironic_client.node.list.called)

        expected_flavor = {'disk': '10', 'ephemeral': '20',
                           'kernel': 'aaa', 'ramdisk': 'bbb',
                           'name': 'baremetal_2048_10_20_1',
                           'memory': '2048', 'arch': 'i386',
                           'cpu': '1'}
        create_flavor.assert_called_once_with(client, expected_flavor)

    def test_check_node_properties(self):
        node = mock.MagicMock()
        properties = {'memory_mb': '1024',
                      'local_gb': '10',
                      'cpus': '1',
                      'cpu_arch': 'i386'}
        node.properties = properties

        self.assertTrue(flavors.check_node_properties(node))

        properties['memory_mb'] = None
        self.assertFalse(flavors.check_node_properties(node))

        del properties['memory_mb']
        self.assertFalse(flavors.check_node_properties(node))



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import sys

import pbr.version


__version__ = pbr.version.VersionInfo('os_cloud_config').version_string()


def configure_logging(args=None):
    if args and args.log_config:
        logging.config.fileConfig(args.log_config,
                                  disable_existing_loggers=False)
    else:
        format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
        date_format = '%Y-%m-%d %H:%M:%S'
        log_level = logging.DEBUG if args and args.debug else logging.INFO
        logging.basicConfig(datefmt=date_format,
                            format=format,
                            level=log_level,
                            stream=sys.stdout)

configure_logging()
LOG = logging.getLogger(__name__)
LOG.warning("os-cloud-config is DEPRECATED in the Ocata release and will "
            "be removed in Pike.")



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import logging
import textwrap

from os_cloud_config.cmd.utils import _clients as clients
from os_cloud_config.cmd.utils import environment
from os_cloud_config import glance


def parse_args():
    description = textwrap.dedent("""
    Uploads the provided kernel and ramdisk to a Glance store.
    """)

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument('-k', '--kernel', dest='kernel',
                        help='Name of the kernel image', required=True)
    parser.add_argument('-l' '--kernel-file', dest='kernel_file',
                        help='Kernel to upload', required=True)
    parser.add_argument('-r', '--ramdisk', dest='ramdisk',
                        help='Name of the ramdisk image', required=True)
    parser.add_argument('-s', '--ramdisk-file', dest='ramdisk_file',
                        help='Ramdisk to upload', required=True)
    environment._add_logging_arguments(parser)
    return parser.parse_args()


def main():
    args = parse_args()
    environment._configure_logging(args)
    try:
        environment._ensure()
        client = clients.get_glance_client()
        glance.create_or_find_kernel_and_ramdisk(
            client, args.kernel, args.ramdisk, kernel_path=args.kernel_file,
            ramdisk_path=args.ramdisk_file)
    except Exception:
        logging.exception("Unexpected error during command execution")
        return 1
    return 0



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import json
import logging
import os
import textwrap

from os_cloud_config.cmd.utils import _clients as clients
from os_cloud_config.cmd.utils import environment
from os_cloud_config import flavors


def parse_args():
    description = textwrap.dedent("""
    Create flavors describing the compute resources the cloud has
    available.

    If the list of flavors is only meant to encompass hardware that the cloud
    has available, a JSON file describing the nodes can be specified, see
    register-nodes for the format.

    If a custom list of flavors is to be created, they can be specified as a
    list of JSON objects. Each list item is a JSON object describing one
    flavor, which has a "name" for the flavor, "memory" in MB, "cpu" in
    threads, "disk" in GB and "arch" as one of i386/amd64/etc.
    """)

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument('-n', '--nodes', dest='nodes',
                       help='A JSON file containing a list of nodes that '
                       'distinct flavors will be generated and created from')
    group.add_argument('-f', '--flavors', dest='flavors',
                       help='A JSON file containing a list of flavors to '
                       'create directly')
    group.add_argument('-i', '--ironic', action='store_true',
                       help='Pull the registered list of nodes from Ironic '
                       'that distinct flavors will be generated and created '
                       'from')
    parser.add_argument('-k', '--kernel', dest='kernel',
                        help='ID of the kernel in Glance', required=True)
    parser.add_argument('-r', '--ramdisk', dest='ramdisk',
                        help='ID of the ramdisk in Glance', required=True)
    environment._add_logging_arguments(parser)
    return parser.parse_args()


def main():
    args = parse_args()
    environment._configure_logging(args)
    try:
        environment._ensure()
        client = clients.get_nova_bm_client()
        flavors.cleanup_flavors(client)
        root_disk = os.environ.get('ROOT_DISK', None)
        if args.nodes:
            with open(args.nodes, 'r') as nodes_file:
                nodes_list = json.load(nodes_file)
            flavors.create_flavors_from_nodes(
                client, nodes_list, args.kernel, args.ramdisk, root_disk)
        elif args.flavors:
            with open(args.flavors, 'r') as flavors_file:
                flavors_list = json.load(flavors_file)
            flavors.create_flavors_from_list(
                client, flavors_list, args.kernel, args.ramdisk)
        elif args.ironic:
            ironic_client = clients.get_ironic_client()
            flavors.create_flavors_from_ironic(
                client, ironic_client, args.kernel, args.ramdisk, root_disk)
    except Exception:
        logging.exception("Unexpected error during command execution")
        return 1
    return 0



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from os_cloud_config.tests import base
from os_cloud_config.utils import clients


class ClientsTest(base.TestCase):

    @mock.patch('ironicclient.client.get_client')
    def test_get_ironic_client(self, client_mock):
        clients.get_ironic_client('username', 'password', 'tenant_name',
                                  'auth_url')
        client_mock.assert_called_once_with(
            1, os_username='username',
            os_password='password',
            os_auth_url='auth_url',
            os_tenant_name='tenant_name',
            ca_file=None)

    @mock.patch('novaclient.client.Client')
    def test_get_nova_bm_client(self, client_mock):
        clients.get_nova_bm_client('username', 'password', 'tenant_name',
                                   'auth_url')
        client_mock.assert_called_once_with('2', 'username',
                                            'password',
                                            'tenant_name',
                                            'auth_url',
                                            cacert=None,
                                            extensions=[mock.ANY])

    @mock.patch('keystoneclient.v2_0.client.Client')
    def test_get_keystone_client(self, client_mock):
        clients.get_keystone_client('username', 'password', 'tenant_name',
                                    'auth_url')
        client_mock.assert_called_once_with(
            username='username',
            password='password',
            auth_url='auth_url',
            tenant_name='tenant_name',
            cacert=None)

    @mock.patch('keystoneclient.v3.client.Client')
    def test_get_keystone_v3_client_with_v2_url(self, client_mock):
        clients.get_keystone_v3_client('username', 'password', 'tenant_name',
                                       'auth_url/v2.0')
        client_mock.assert_called_once_with(
            username='username',
            password='password',
            auth_url='auth_url/v3',
            tenant_name='tenant_name',
            cacert=None)

    @mock.patch('keystoneclient.v3.client.Client')
    def test_get_keystone_v3_client_with_v3_url(self, client_mock):
        clients.get_keystone_v3_client('username', 'password', 'tenant_name',
                                       'auth_url/v3')
        client_mock.assert_called_once_with(
            username='username',
            password='password',
            auth_url='auth_url/v3',
            tenant_name='tenant_name',
            cacert=None)

    @mock.patch('neutronclient.neutron.client.Client')
    def test_get_neutron_client(self, client_mock):
        clients.get_neutron_client('username', 'password', 'tenant_name',
                                   'auth_url')
        client_mock.assert_called_once_with(
            '2.0', username='username',
            password='password',
            auth_url='auth_url',
            tenant_name='tenant_name',
            ca_cert=None)

    @mock.patch('keystoneclient.session.Session')
    @mock.patch('keystoneclient.auth.identity.v2.Password')
    @mock.patch('glanceclient.Client')
    def test_get_glance_client(self, client_mock, password_mock, session_mock):
        clients.get_glance_client('username', 'password', 'tenant_name',
                                  'auth_url')
        password_mock.assert_called_once_with(auth_url='auth_url',
                                              username='username',
                                              password='password',
                                              tenant_name='tenant_name')
        session_mock.assert_called_once_with(auth=password_mock.return_value)
        session_mock.return_value.get_endpoint.assert_called_once_with(
            service_type='image', interface='public', region_name='regionOne')
        session_mock.return_value.get_token.assert_called_once_with()
        client_mock.assert_called_once_with(
            '1', endpoint=session_mock.return_value.get_endpoint.return_value,
            token=session_mock.return_value.get_token.return_value,
            cacert=None)



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

import mock

from os_cloud_config.cmd import upload_kernel_ramdisk
from os_cloud_config.tests import base


class UploadKernelRamdiskTest(base.TestCase):

    @mock.patch('os_cloud_config.cmd.utils._clients.get_glance_client',
                return_value='glance_client_mock')
    @mock.patch('os_cloud_config.glance.create_or_find_kernel_and_ramdisk')
    @mock.patch.dict('os.environ', {'OS_USERNAME': 'a', 'OS_PASSWORD': 'a',
                     'OS_TENANT_NAME': 'a', 'OS_AUTH_URL': 'a'})
    @mock.patch.object(sys, 'argv', ['upload_kernel_ramdisk', '-k',
                                     'bm-kernel', '-r', 'bm-ramdisk', '-l',
                                     'kernel-file', '-s', 'ramdisk-file'])
    def test_with_arguments(self, create_or_find_mock, glanceclient_mock):
        upload_kernel_ramdisk.main()
        create_or_find_mock.assert_called_once_with(
            'glance_client_mock', 'bm-kernel', 'bm-ramdisk',
            kernel_path='kernel-file', ramdisk_path='ramdisk-file')



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

import glanceclient
from ironicclient import client as ironicclient
from keystoneclient.auth.identity import v2
from keystoneclient import session
from keystoneclient.v2_0 import client as ksclient
from keystoneclient.v3 import client as ks3client
from neutronclient.neutron import client as neutronclient
from novaclient import client as nova_client
from novaclient.extension import Extension
from novaclient.v2.contrib import baremetal

LOG = logging.getLogger(__name__)


def get_nova_bm_client(username, password, tenant_name, auth_url, cacert=None):
    LOG.debug('Creating nova client.')
    baremetal_extension = Extension('baremetal', baremetal)
    return nova_client.Client("2", username, password, tenant_name, auth_url,
                              extensions=[baremetal_extension], cacert=cacert)


def get_ironic_client(username, password, tenant_name, auth_url, cacert=None):
    LOG.debug('Creating ironic client.')
    kwargs = {'os_username': username,
              'os_password': password,
              'os_auth_url': auth_url,
              'os_tenant_name': tenant_name,
              'ca_file': cacert}

    return ironicclient.get_client(1, **kwargs)


def get_keystone_client(username,
                        password,
                        tenant_name,
                        auth_url,
                        cacert=None):

    LOG.debug('Creating keystone client.')
    kwargs = {'username': username,
              'password': password,
              'tenant_name': tenant_name,
              'auth_url': auth_url,
              'cacert': cacert}

    return ksclient.Client(**kwargs)


def get_keystone_v3_client(username,
                           password,
                           tenant_name,
                           auth_url,
                           cacert=None):

    LOG.debug('Creating keystone v3 client.')
    kwargs = {'username': username,
              'password': password,
              'tenant_name': tenant_name,
              'auth_url': auth_url.replace('v2.0', 'v3'),
              'cacert': cacert}

    return ks3client.Client(**kwargs)


def get_neutron_client(username,
                       password,
                       tenant_name,
                       auth_url,
                       cacert=None):
    LOG.debug('Creating neutron client.')
    kwargs = {'username': username,
              'password': password,
              'tenant_name': tenant_name,
              'auth_url': auth_url,
              'ca_cert': cacert}

    neutron = neutronclient.Client('2.0', **kwargs)
    neutron.format = 'json'
    return neutron


def get_glance_client(username, password, tenant_name, auth_url, cacert=None,
                      region_name='regionOne'):
    LOG.debug('Creating Keystone session to fetch Glance endpoint.')
    auth = v2.Password(auth_url=auth_url, username=username, password=password,
                       tenant_name=tenant_name)
    ks_session = session.Session(auth=auth)
    endpoint = ks_session.get_endpoint(service_type='image',
                                       interface='public',
                                       region_name=region_name)
    token = ks_session.get_token()
    LOG.debug('Creating glance client.')
    return glanceclient.Client('1', endpoint=endpoint, token=token,
                               cacert=cacert)



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import logging
import textwrap

import os_cloud_config.cmd.utils._clients as clients
from os_cloud_config.cmd.utils import environment
from os_cloud_config.keystone import initialize_for_heat


def parse_args():
    description = textwrap.dedent("""
    Create a domain for Heat to use, as well as a user to administer it.

    This will create a heat domain in Keystone, as well as an admin user that
    has rights to administer the domain.
    """)
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=description)
    parser.add_argument('-d', '--domain-admin-password',
                        dest='domain_admin_password',
                        help="domain admin user's password to be set",
                        required=True)
    environment._add_logging_arguments(parser)
    return parser.parse_args()


def main():
    args = parse_args()
    environment._configure_logging(args)
    try:
        environment._ensure()
        keystone_client = clients.get_keystone_v3_client()
        initialize_for_heat(keystone_client, args.domain_admin_password)
    except Exception:
        logging.exception("Unexpected error during command execution")
        return 1
    return 0
